	   .386P
	   .LIST
	   PAGE 55,132
;-------------------------------------------------------------------------+
;									  !
; (c) Copyright, Wang Laboratories, Inc., 1989				  !
;     All rights reserved.						  !
;									  !
; PROGRAM NAME: WANG 2200 CS386 Diagnostics				  !
;									  !
; FILE NAME:	CSDG.SRC						  !
;									  !
; SUPPORT GROUP: Engineering Service Department 			  !
;		 Wang Computer (Taiwan) LTD.				  !
;		 2, Science Road One	   ZIP 20077			  !
;		 Hsinchu, Taiwan, Republic of China			  !
;									  !
; REVISION HISTORY:							  !
; DATE	       PROGRAMMER      DESCRIPTION				  !
;  6-13-1989   C.C. Mao        R1960  for PCA R0M0 onward		  !
;									  !
; Assumptions upon download this program to CS386 CPU memory:		  !
; 1. Base of CS, DS, ES, SS are 160H. FS's base is FFFF8000H.		  !
; 2. Upon exit, use GDT at 0H, jump to 0FFE90034H (EPROM)		  !
;									  !
;									  !
; TEST ROUTINE: 							  !
;[Test 1]  RAM Data Bus Test						  !
; Purpose: To verify that no shorts or opens exist on RAM data bus.	  !
;	   Some faults with the RAM parity logic will be detected.	  !
;									  !
;[Test 2]  RAM Parity Generator/Check Test				  !
; Purpose: To verify RAM parity error circuity. 			  !
;									  !
;[Test 3]  RAM Byte Decode Address Line Test				  !
; Purpose: To check for shorts and opens on the RAM byte decode address   !
;	   lines.							  !
;									  !
;[Test 4]  RAM Address Line Test					  !
; Purpose: To verify RAM address lines and check for shorts and opens on  !
;	   the RAM address bus and internal RAM chip addressing.	  !
;									  !
;[Test 5]  RAM Data Fast Exchange Function TEST 			  !
;Purpose:  To test ability of all RAM memory cells to hold a zero or a	  !
;	   one and detec dynamic data/address faults.			  !
;									  !
;									  !
;-------------------------------------------------------------------------+
GDTBASE EQU	00001000H	;phisical address for GDT Base

;I/O PORT
TIMESET EQU	     8000H	;120NS FOR EACH UNIT

CPUST	EQU		2H	;READ CPU/IO STATUS
				;D0= 1, 5us pulse of ABS/, OBS/, CBS/
				;D1= 1, CPU BUSY
				;D2= 1, KEY BOARD FUNCTION
				;D3= 1, I/O READY
				;D4= 1, TIME OUTME OUT
				;D5= 1, CPU HALT
				;D6= 1, SRAM PARITY ERROR
				;D7= 1, DRAM PARITY ERROR
				;D8= 1, primary reset in from 2200 bus
				;D9= 1, nomal boot; =0, run-in
				;D10=1, same as D2, but read cause clear
				;D11	not defined
				;D12	not defined
				;D13=1, same as D5, but read cause clear

CPUFLG	EQU		4H	;WRITE IO PORT
				;D0= 1, generate even parity bit for diag
				;  = 0, clear force even parity condition
				;D1= 1, clear parity error if occured anytime
				;  = 0, not clear parity error, if occured
				;D2= 1, clear 387 error if occured
				;  = 0, not clear 387 error if occured
				;D3= 1, CPU LED on
				;  = 0, CPU LED off

CPURDY	EQU	       6H	;release CPU BUSY

RTCLOCK EQU	       8H	;realtime clock address strobe
RTCLOC2 EQU	      0AH	;realtime clock data strobe, data r/w port
				;1st byte RTC user RAM = memory size in MB
				;17byte user ram = error code in protect mode
				;18th byte = MNI flag; 0= no NMI, 1=NMI occured
				;19th byte = loop count low byte
				;20th byte = loop count hi byte

IBGET	EQU	       0CH	;get input data from bus
_ABS	EQU	       12H	;address bus strobe
_OBS	EQU	       14H	;output data to bus
CBS	EQU	       18H	;command data bus strobe

CODE	SEGMENT USE32
	ASSUME CS:CODE,DS:CODE,ES:CODE

START:
	CLD
	MOV	EDX,OFFSET STACK_BUFFER
	ADD	EDX,200H	; extend minimun 100H spac for stack
	AND	EDX,0FFFFFF00H	; get first address to test
	MOV	FIRSTRAM,EDX	; save this address
	MOV	ESP,EDX 	; and set up stack point

	PUSH	DS		; save current ds
	MOV	EAX,0		; clear its
	MOV	EBX,EAX
	MOV	AX,0CH		; tempery point to LDT
	MOV	DS,AX		;
	MOV	AX,CS		; calculate which one discriptor we use now
	AND	AX, 0FFFbH	; mask out privalige
	MOV	BX,[EAX+2]	; get base address
	POP	DS		; restore original ds

	MOV	BASE,EBX	; save this ds base offset
	MOV	EDX,10000H	; try to find out physical 64KB boundery address
BANK64K:
	CMP	ESP,EDX 	; does it big than current EDX?
	JNG	BANK64KA	; yes, this program is within current edx size
	ADD	EDX,10000H	; otherwise try next 64KB boundry
	JMP	BANK64K
BANK64KA:
	SUB	EDX,EBX 	; deduct base offset from descriptor
	MOV	BOUNDRY,EDX	; save this boundry address for following tests
	MOV	BOUNDRYA,DX	; save this low word for following tests

	MOV	AX,2000H	; RTC RAM 20h contains loopcount low byte
	CALL	WRITERTC
	MOV	AX,2100H	; RTC RAM 21h contains loopcount hi byte
	CALL	WRITERTC	; reset loopcount

SHOWLOOP:
	MOV	AL,5		; CRT device address is 05h
	CALL	ENABLE		; now display some messages on MXE ch0
	MOV	ESI,OFFSET COPYRIGHT
	MOV	ECX,AFTER_COPYRIGHT - COPYRIGHT
	CALL	DISPLAYMORE	; display copyright statement

;-------------------------------------------------------------------------+
;[Test 1]  RAM Data Bus Test						  !
;									  !
; Purpose: To verify that no shorts or opens exist on RAM data bus.	  !
;	   Some faults with the RAM parity logic will be detected.	  !
;	   It also stores memory size at 1st byte of RTC user RAM.	  !
;									  !
; Description: Walking one and zero data patterns (32-bit) are written,   !
;	   then read/verified for correct data and NO parity error, on	  !
;	   bank basis. All patterns has an odd number of 'high'(1) bits.  !
;	   A double-bit rotating pattern, starting value '00000003'H, will!
;	   be written/read/verified for correct data and NO parity. These !
;	   patterns have an even number of 'high'(1) bits.		  !
;	   The routine uses address zero of RAM of each 64KB bank,	  !
;	   so chip faults at that location will be flagged as well.	  !
;	   At each end of MB size, store the memory size to RTC.	  !
;									  !
; BEGIN 								  !
;     FOR current bank = Boundry, Boundry+10000H, ........, last bank	  !
;	  current address = first in current bank			  !
;	 FOR current pattern = FFFFFFFF, 01, 02, 04, ...80000000, 0,	  !
;			       7FFFFFFF, BFFFFFFF, ......., FFFFFFFE	  !
;	     write current pattern to current address			  !
;	     read/verify current address		ERROR 20	  !
;	     check for memory parity error		ERROR 21	  !
;	 NEXT pattern							  !
;     NEXT bank 							  !
;									  !
;     FOR current bank = Boundry, Boundry+10000H, ........, last bank	  !
;	  current address = first in current bank			  !
;	 FOR current pattern = 03, 06, 0C, 18, 30,...., C0000000, 80000001!
;	     write current pattern to current address			  !
;	     read/verify current address		ERROR 22	  !
;	     check for memory parity error		ERROR 23	  !
;	 NEXT pattern							  !
;     NEXT bank 							  !
; END									  !
;									  !
; ERRORS:								  !
; 20 & 22 Data bus or memory data error 				  !
;	  EAX= read data    EBX= expected data	   DS:SI= current address !
;									  !
; 21 & 23 Memory error, may be parity or circuit			  !
;	  AX= 4, SRAM parity err					  !
;-------------------------------------------------------------------------+
TST1:
	MOV	ESI,OFFSET TST0NAME
	MOV	ECX,TST0NAMEND-TST0NAME ;count bytes
	CALL	DISPLAYMORE	; point cursor to test name
	MOV	ESI,OFFSET TST1NAME
	MOV	ECX,35		; max 35 bytes in string
	CALL	DISPLAYMORE	; display test 1 name


	MOV	ESI,BOUNDRY	; start from lowest 64KB boundry
TST1LOP1:
	MOV	EBX,0FFFFFFFFH	; current data pattern
	MOV	EAX,01		; next pattern
	MOV	EDX,EAX 	; save next pattern
	MOV	ECX,33		; 33 patterns from FFFFFFFF to 01h
	MOV	[ESI],EBX	; write current pattern to current address

TST1LOP2:
	XCHG	[ESI],EAX	; read data from currnt addr & write next
	CMP	EAX,EBX 	; verify read data against current pattern
	JNE	TST1SIZE	; jump if bad (no jump is fast)

	MOV	EBX,EDX 	; save current pattern
	SHL	EDX,1		; setup next data pattern
	MOV	EAX,EDX 	; ready to write data pattern from EAX reg
	LOOP	TST1LOP2	; loop for next pattern test

	MOV	EAX,7FFFFFFFH	; next data pattern 7FFFFFFF
	MOV	EDX,EAX 	; prepare for shift right
	XCHG	[ESI],EAX	; read data from currnt addr & write next
	CMP	EAX,EBX 	; verify read data against current pattern
	JNE	TST1SIZE	 ; jump if bad (no jump is fast)

	MOV	EBX,EDX 	; save current pattern for compare later
	ROR	EDX,1		; setup next data pattern
	MOV	EAX,EDX
	MOV	CX,31		; 31 patterns from 7FFFFFFF to FFFFFFFE

TST1LOP3:
	XCHG	[ESI],EAX	; read data from currnt addr & write next
	CMP	EAX,EBX 	; verify read data against current pattern
	JNE	TST1SIZE	 ; jump if bad (no jump is fast)

	MOV	EBX,EDX 	; save current pattern for compare later
	ROR	EDX,1		; setup next data pattern
	MOV	EAX,EDX
	LOOP	TST1LOP3	; loop for next pattern test

	MOV	EDI,CPUST	; point to CPU status port
	TEST	BYTE PTR FS:[EDI], 11000000B ; is D6 or D7 = 1?
	JNE	TST2ERR 	; yes, all zero; no parrity error

	ADD	ESI,10000H	; point to next 64KB bank
	CALL	LASTADDR	; get last dram address in edx
	CMP	ESI,EDX 	;
	JL	TST1LOP1	;

TST1SIZE:
	MOV	ECX,BASE
	SUB	ECX,1
	TEST	ESI,ECX 	; Is this address within 64K bank?
	JNZ	TST1ERR 	; No, it is not 1st byte on 64KB
	MOV	ECX,ESI
	CMP	CX,BOUNDRYA	; Is this 1st address of each MB?
	JNZ	TST1ERR 	; No, it is not 1st byte on MB
	TEST	ESI,030000H	; Is this 1st address of each MB?
	JZ	TST1ERR 	; No, it is not 1st byte on MB
	MOV	EAX,ESI 	; Good, this address is 1st byte
	SHR	EAX,20		; scratch out low 20 bits
	TEST	EAX,0FFh	; make sure the DRAM existing
	JZ	TST1ERR 	; bad, it is 1st byte of 1MB
	TEST	EAX,1		; DRAM odd size?
	JNZ	TST1SIZEX	; yes, it is odd MB size
TST1SIZEOK:
	MOV	BL,AL		; save the DRAM MB size
	MOV	AH,0Eh		; point to 1st byte on user RAM
	CALL	READRTC 	; get MB size read by EPROM diag
	CMP	AL,BL		; they should be equal
	JNE	TST1ERR 	; if not equal, something wrong

	JMP	TST2		; go on next test

TST1SIZEX:
	CMP	EAX,1		; DRAM = 1MB size?
	JE	TST1SIZEOK	; 1MB size is legal

TST1ERR:
	MOV	EBP,EAX 	; save read data
	MOV	ECX,8
TST1ERR1:
	AND	EAX,0FH 	; mask out all hi nibble
	LEA	EDI,ASCIIFONT	; point to ascii string
	MOV	AH,[EDI+EAX]	; exchange hex to ascii
	MOV	READDATA[ECX],AH; put it to display string
	SHR	EBP,4		; take low nibble out
	MOV	EAX,EBP 	; convert next nibble
	LOOP	TST1ERR1

	MOV	EBP,EBX 	; save expected data
	MOV	ECX,8
TST1ERR2:
	AND	EBX,0FH 	; mask out all hi nibble
	LEA	EDI,ASCIIFONT	; point to ascii string
	MOV	AH,[EDI+EBX]	; exchange hex to ascii
	MOV	EXPECDATA[ECX],AH; put it to display string
	SHR	EBP,4		; take low nibble out
	MOV	EBX,EBP 	; convert next nibble
	LOOP	TST1ERR2

	MOV	EBX,ESI 	; save expected data
	MOV	ECX,8
TST1ERR3:
	AND	EBX,0FH 	; mask out all hi nibble
	LEA	EDI,ASCIIFONT	; point to ascii string
	MOV	AH,[EDI+EBX]	; exchange hex to ascii
	MOV	ADDRESS[ECX],AH ; put it to display string
	SHR	ESI,4		; take low nibble out
	MOV	EBX,ESI 	; convert next nibble
	LOOP	TST1ERR3

	MOV	ESI,OFFSET ERRMSG
	MOV	ECX,ERRMSGEND - ERRMSG
	CALL	DISPLAYMORE	; display error message

TST1ERR9:
	MOV	ESI,OFFSET SHIFTKEY
	MOV	ECX,RESETKEY - SHIFTKEY
	CALL	DISPLAYMORE	; memtion user by blinking words
	CALL	DELAYSEC
	MOV	ESI,OFFSET RESETKEY
	MOV	ECX,RESETKEYEND - RESETKEY
	CALL	DISPLAYMORE	; memtion user by blinking words
	CALL	DELAYSEC
	JMP	TST1ERR9	; loop here wait for reset key pressed


;-------------------------------------------------------------------------+
;[Test 2] DRAM Parity Generator/Check Test				  !
;									  !
; Purpose: To verify DRAM parity error circuity.			  !
;									  !
; Description: The test first writes a 32-bit pattern with an odd number  !
;	   of bits 'high'(1) in each byte and normal parity generation to !
;	   the lowest address in the bank. The data is then read/verified !
;	   and parity status checked for NO error. The FORCED parity fun- !
;	   ction is enabled which causes the EVEN parity logic to generate!
;	   a 'high'(1) on the four outputs, then read/verified again, with!
;	   a check to ensure parity error occurred. The FORCED parity fun-!
;	   ction is then disabled and the parity error is reset. The pat- !
;	   tern is then rotated and the algorithm is repeated within each !
;	   byte to test all the lines to the parity generator/checker	  !
;	   chips. Although 32-bit writes are performed in this test, each !
;	   byte is read back for test separately, starting from the least !
;	   significant byte and processing to the most significant byte   !
;	   of the 4-byte double-word.					  !
;									  !
; BEGIN 								  !
;     FOR current bank = DRAM						  !
;	  current address = first in current bank			  !
;	 FOR current pattern = 01010101h, 02020202h, 04040404h, .....	  !
;			       20202020h, 40404040h, 80808080h, 	  !
;	     write current pattern to current address			  !
;	     read/verify current address		ERROR 24	  !
;	     check for memory no parity error		ERROR 25	  !
;	     enable forced parity function				  !
;	     read/verify current address		ERROR 26	  !
;	     check for memory parity error occurred	ERROR 27	  !
;	     disable forced parity function				  !
;	 NEXT pattern							  !
;     NEXT bank 							  !
; END									  !
;									  !
; ERRORS:								  !
; 24 & 26 Data bus or memory data error 				  !
;	  EAX= read data    EBX= expected data	   DS:SI= current address !
;	  17th byte of RTC user RAM = error code			  !
;									  !
; 25 & 27 Memory error, may be parity or circuit			  !
;	  AX= 80,DRAM parity err					  !
;	  17th byte of RTC user RAM = error code			  !
;-------------------------------------------------------------------------+

TST2:
	CALL	DELAYSEC
	MOV	ESI,OFFSET TST0NAME
	MOV	ECX,TST0NAMEND-TST0NAME ;count bytes
	CALL	DISPLAYMORE	; point cursor to test name
	MOV	ESI,OFFSET TST2NAME
	MOV	ECX,35		; max 35 bytes in string
	CALL	DISPLAYMORE	; display test 2 name

				; disable NMI first
	MOV	DX,0		; D0= 0, normal parity generate
				; D1= 0, not clear parity error
				; D6= 0, disable NMI
	MOV	FS:[EDI],DX	; expect parity error at next read

	MOV	ESI,BOUNDRY	; start from lowest 64KB boundry
	MOV	EBP,CPUST	; FS:[EBP]= CPU status port
	MOV	EDI,CPUFLG	; FS:[EDI]= CPU flag I/O port

TST2LOPA:
	MOV	EBX,01010101H	; current data pattern, odd '1'
TST2LOP1:
	MOV	[ESI],EBX	; write data to current address
	MOV	EAX,0
	MOV	ECX,4		; we have 4 bytes to read
TST2LOP2:
	MOV	AL,[ESI]	; read data from currnt addr first byte
	CMP	AL,BL		; verify read data against current pattern
	JNE	TST1ERR 	; jump if bad (no jump is fast)

	TEST	BYTE PTR FS:[EBP],0C0H ; keep D7 and D6, mask out other bits
	JNZ	TST2ERR 	; if D7 not zero; parrity error ocurred

	INC	ESI
	LOOP	TST2LOP2

	MOV	DX,01H		; D0= 1, force parity generate error
	MOV	FS:[EDI],DX	; we expect parity error at next read
	MOV	SI,BOUNDRYA	; start from lowest 64KB boundry
	MOV	[ESI],EBX	; write in order to get wrong parity
	MOV	ECX,4		; we have 4 bytes to read
TST2LOP3:
	MOV	AL,[ESI]	; read data from currnt addr first byte
	CMP	AL,BL		; verify read data against current pattern
	JNE	TST1ERR 	; jump if bad (no jump is fast)

	TEST	BYTE PTR FS:[EBP],0C0H ; keep D7 and D6, mask out other bits
	JZ	TST2ERRA	; if D7 or D6 not zero; parrity error ocurred

	MOV	DX,02H		; D1= 1, clear parity error
	MOV	FS:[EDI],DX	; expect parity error at next read
	TEST	BYTE PTR FS:[EBP],0C0H ; keep D7 and D6, mask out other bits
	JNZ	TST2ERRB	; D7 or D6 not zero; parrity error can't reset
	MOV	DX,0		; D1= 0, not clear parity error
				; D0= 0, normal parity generate
	MOV	FS:[EDI],DX	; expect parity error at next read

	INC	ESI
	LOOP	TST2LOP3

	MOV	SI,BOUNDRYA	; start from lowest 64KB boundry
	SHL	EBX,1		; setup next data pattern
	JNC	TST2LOP1	; if carry, all odd '1' pattern passed

	ADD	ESI,10000H	; try next 64KB boundry
	CALL	LASTADDR	; get last address in EDX
	CMP	ESI,EDX 	; do we reach last address?
	JL	TST2LOPA	; if not, test next address
;------------------------ without IDT and NMI service routine, don't enable it +
;	MOV	DX,40H		; D0= 0, normal parity generate 	       ;
;				; D1= 0, not clear parity error 	       ;
;				; D6= 0, disable NMI			       ;
;	MOV	FS:[EDI],DX	; reset to normal parity with NMI enabled      ;
;------------------------------------------------------------------------------+
	JMP	TST3		; end of parity circult testing

TST2ERR:
	MOV	EBP,EAX 	; save read data
	MOV	ECX,8
TST2ERR1:
	AND	EAX,0FH 	; mask out all hi nibble
	LEA	EDI,ASCIIFONT	; point to ascii string
	MOV	AH,[EDI+EAX]	; exchange hex to ascii
	MOV	READDATA[ECX],AH; put it to display string
	SHR	EBP,4		; take low nibble out
	MOV	EAX,EBP 	; convert next nibble
	LOOP	TST2ERR1

	MOV	EBP,EBX 	; save expected data
	MOV	ECX,8
TST2ERR2:
	AND	EBX,0FH 	; mask out all hi nibble
	LEA	EDI,ASCIIFONT	; point to ascii string
	MOV	AH,[EDI+EBX]	; exchange hex to ascii
	MOV	EXPECDATA[ECX],AH; put it to display string
	SHR	EBP,4		; take low nibble out
	MOV	EBX,EBP 	; convert next nibble
	LOOP	TST2ERR2

	MOV	EBX,ESI 	; save expected data
	MOV	ECX,8
TST2ERR3:
	AND	EBX,0FH 	; mask out all hi nibble
	LEA	EDI,ASCIIFONT	; point to ascii string
	MOV	AH,[EDI+EBX]	; exchange hex to ascii
	MOV	ADDRESS[ECX],AH; put it to display string
	SHR	ESI,4		; take low nibble out
	MOV	EBX,ESI 	; convert next nibble
	LOOP	TST2ERR3

	MOV	ESI,OFFSET ERRMSG
	MOV	ECX,ERRMSGEND - ERRMSG
	CALL	DISPLAYMORE	; display error message
	MOV	ESI,OFFSET TST2MSG
	MOV	ECX,TST2MSGA - TST2MSG
	CALL	DISPLAYMORE	; display error message
	JMP	TST1ERR9	; wait reset key

TST2ERRA:
	MOV	EBP,EAX 	; save read data
	MOV	ECX,8
TST2ERR1A:
	AND	EAX,0FH 	; mask out all hi nibble
	LEA	EDI,ASCIIFONT	; point to ascii string
	MOV	AH,[EDI+EAX]	; exchange hex to ascii
	MOV	READDATA[ECX],AH; put it to display string
	SHR	EBP,4		; take low nibble out
	MOV	EAX,EBP 	; convert next nibble
	LOOP	TST2ERR1A

	MOV	EBP,EBX 	; save expected data
	MOV	ECX,8
TST2ERR2A:
	AND	EBX,0FH 	; mask out all hi nibble
	LEA	EDI,ASCIIFONT	; point to ascii string
	MOV	AH,[EDI+EBX]	; exchange hex to ascii
	MOV	EXPECDATA[ECX],AH; put it to display string
	SHR	EBP,4		; take low nibble out
	MOV	EBX,EBP 	; convert next nibble
	LOOP	TST2ERR2A

	MOV	EBX,ESI 	; save expected data
	MOV	ECX,8
TST2ERR3A:
	AND	EBX,0FH 	; mask out all hi nibble
	LEA	EDI,ASCIIFONT	; point to ascii string
	MOV	AH,[EDI+EBX]	; exchange hex to ascii
	MOV	ADDRESS[ECX],AH; put it to display string
	SHR	ESI,4		; take low nibble out
	MOV	EBX,ESI 	; convert next nibble
	LOOP	TST2ERR3A

	MOV	ESI,OFFSET ERRMSG
	MOV	ECX,ERRMSGEND - ERRMSG
	CALL	DISPLAYMORE	; display error message
	MOV	ESI,OFFSET TST2MSGA
	MOV	ECX,TST2MSGB - TST2MSGA
	CALL	DISPLAYMORE	; display error message
	JMP	TST1ERR9	; wait reset key

TST2ERRB:
	MOV	EBP,EAX 	; save read data
	MOV	ECX,8
TST2ERR1B:
	AND	EAX,0FH 	; mask out all hi nibble
	LEA	EDI,ASCIIFONT	; point to ascii string
	MOV	AH,[EDI+EAX]	; exchange hex to ascii
	MOV	READDATA[ECX],AH; put it to display string
	SHR	EBP,4		; take low nibble out
	MOV	EAX,EBP 	; convert next nibble
	LOOP	TST2ERR1B

	MOV	EBP,EBX 	; save expected data
	MOV	ECX,8
TST2ERR2B:
	AND	EBX,0FH 	; mask out all hi nibble
	LEA	EDI,ASCIIFONT	; point to ascii string
	MOV	AH,[EDI+EBX]	; exchange hex to ascii
	MOV	EXPECDATA[ECX],AH; put it to display string
	SHR	EBP,4		; take low nibble out
	MOV	EBX,EBP 	; convert next nibble
	LOOP	TST2ERR2B

	MOV	EBX,ESI 	; save expected data
	MOV	ECX,8
TST2ERR3B:
	AND	EBX,0FH 	; mask out all hi nibble
	LEA	EDI,ASCIIFONT	; point to ascii string
	MOV	AH,[EDI+EBX]	; exchange hex to ascii
	MOV	ADDRESS[ECX],AH ; put it to display string
	SHR	ESI,4		; take low nibble out
	MOV	EBX,ESI 	; convert next nibble
	LOOP	TST2ERR3B

	MOV	ESI,OFFSET ERRMSG
	MOV	ECX,ERRMSGEND - ERRMSG
	CALL	DISPLAYMORE	; display error message
	MOV	ESI,OFFSET TST2MSGB
	MOV	ECX,TST2MSGEND - TST2MSGB
	CALL	DISPLAYMORE	; display error message
	JMP	TST1ERR9	; wait reset key

;-------------------------------------------------------------------------+
;[Test 3] DRAM Byte Decode Address Line Test				  !
;									  !
; Purpose: To check for shorts and opens on the DRAM byte decode address  !
;	   lines.							  !
;									  !
; Description: Data is writen and verified to the beginning of each bank  !
;	   in byte and word patterns. Doubleword patterns have been tested!
;	   in the [Test 6] DRAM Data Line Test. The 80386 stores word data!
;	   in memory in reverse byte order i.e. the high-order byte follo-!
;	   wing the low-order byte. Therefore the high (most-significant) !
;	   byte of a data word is in the highest memory address. The word !
;	   read/write can be performed at both even and odd address.	  !
;									  !
; BEGIN 								  !
;     FOR current bank = DRAM						  !
;	  write 00h, 01h, 02h, 03h for the offset addresses 00h, 01h, 02h,!
;		03h respectively.					  !
;	  read/verify data 00h, 01h, 02h, 03h for the offset addresses	  !
;		00h, 01h, 02h, 03h respectively.	     ERROR 28	  !
;	  write 03h, 02h, 01h, 00h for the offset addresses 00h, 01h, 02h,!
;		03h respectively.					  !
;	  read/verify data 03h, 02h, 01h, 00h for the offset addresses	  !
;		00h, 01h, 02h, 03h respectively.	     ERROR 29	  !
;	  write AA55h, CC33h for the offset address 00h, 02h respectively.!
;	  read/verify AA55h, CC33h for offset addresses 00h, 02h	  !
;		respectively.				     ERROR 2A	  !
;	  write 9966h for offset address 01h.				  !
;	  read/verify 6655h, 9966h, CC99h for offset addresses 00h, 01h,  !
;		02h respectively.			     ERROR 2B	  !
;     NEXT bank 							  !
; END									  !
;									  !
; ERRORS:								  !
; 28 & 29,  byte decode address error					  !
;	  AL= read data,   BX & CX=  expected data,  DS:SI= current addres!
;	  17th byte of RTC user RAM = error code			  !
; 2A  word decode address error at even boundry 			  !
;	  AX= read data,   BX & CX=  expected data,  DS:SI= current addres!
;	  17th byte of RTC user RAM = error code			  !
; 2B  word decode address error at odd boundry				  !
;	  AX= read data,   BX, CX & DX=  expected data,  DS:SI= current   !
;	  17th byte of RTC user RAM = error code		address   !
;-------------------------------------------------------------------------+
TST3:
	CALL	DELAYSEC
	MOV	ESI,OFFSET TST0NAME
	MOV	ECX,TST0NAMEND-TST0NAME ;count bytes
	CALL	DISPLAYMORE	; point cursor to test name
	MOV	ESI,OFFSET TST3NAME
	MOV	ECX,35		; max 35 bytes in string
	CALL	DISPLAYMORE	; display test 3 name

	CALL	LASTADDR	; get last address in EDX
	MOV	EBP,EDX 	; store it in EBP; EDX has other use
	MOV	ESI,BOUNDRY	; set lowest 64KB boundry to test

TST3LOP1:
	MOV	CX,0302H	;
	MOV	BX,CX
	SHL	EBX,16		; save it in hi word for error msg
	MOV	BX,0100H	; byte patterns 00h, 01h, 02h, 03h
TST3LOP2:
	MOV	[ESI],BL	; write byte 00h to address 0000h
	MOV	[ESI+1],BH	; write byte 01h to address 0001h
	MOV	[ESI+2],CL	; write byte 02h to address 0002h
	MOV	[ESI+3],CH	; write byte 03h to address 0003h
	MOV	AL,[ESI]	; read byte data from addrs 0000h
	CMP	AL,BL		; verify written byte data
	JNZ	TST3ERR
	INC	ESI		; point to next byte address
	MOV	AL,[ESI]	; read byte data from addrs 0001h
	CMP	AL,BH		; verify written byte data
	JNZ	TST3ERR
	INC	ESI		; point to next byte address
	MOV	AL,[ESI]	; read byte data from addrs 0002h
	CMP	AL,CL		; verify written byte data
	JNZ	TST3ERR
	INC	ESI		; point to next byte address
	MOV	AL,[ESI]	; read byte data from addrs 0003h
	CMP	AL,CH		; verify written byte data
	JNZ	TST3ERR

	MOV	EDI,CPUST	; point to CPU status port
	TEST	BYTE PTR FS:[EDI], 11000000B ; is D6 or D7 = 1?
	JNE	TST2ERR 	; yes, all zero; no parrity error

	MOV	SI,BOUNDRYA	; reset offset
	CMP	AL,0		; are we first time here?
	JE	TST30		; yes, jump to word patterns test
	MOV	CX,0001H	;
	MOV	BX,CX
	SHL	EBX,16		; save it in hi word for err msg
	MOV	BX,0203H	; byte patterns 03h, 02h, 01h, 00h
	JMP	TST3LOP2

TST3ERR:
	JMP	TST1ERR 	; show msg and wait reset key

TST30:
	ADD	ESI,10000H	; point to next 64KB bank
	CMP	ESI,EDX 	; last DRAM address is in EDX
	JL	TST3LOP1	; otherwise go on next test

	MOV	ESI,BOUNDRY	; word pattern test from first unused memory
	MOV	EAX,0
TST3LOP3:
	MOV	CX,0CC33H	;
	MOV	BX,CX
	SHL	EBX,16		; save it in hi word for err msg
	MOV	BX,0AA55H	; word patterns 55, AA, 33, CC,
	MOV	[ESI],BX	; write word 55, AA to addrs 00h
	MOV	[ESI+2],CX	; write word 33, CC to addrs 02h
	MOV	AX,[ESI]	; read word data from addrs 0000h
	CMP	AX,BX		; verify written word data
	JNZ	TST3ERR
	ADD	ESI,2		; point to next word address
	MOV	AX,[ESI]	; read word data from addrs 0002h
	CMP	AX,CX		; verify written word data
	JNZ	TST3ERR

	MOV	EDX,0CC99H	; 9966 and CC99.
	MOV	BX,DX		; save it in hi word for err msg
	SHL	EBX,16		; new data in byte series are 55, 66,
	MOV	BX,6655H	; 99, CC; in word series are 6655,
	MOV	CX,9966H	; 9966 and CC99

	MOV	SI,BOUNDRYA	; reset offset
	MOV	[ESI+1],CX	; write word to address 0001h
	MOV	AX,[ESI]	; read word data from addrs 0000h
	CMP	AX,BX		; verify written word data
	JNZ	TST3ERR
	INC	ESI		; point to odd address for word
	MOV	AX,[ESI]	; read word data from addrs 0001h
	CMP	AX,CX		; verify written word data
	JNZ	TST3ERR
	INC	ESI		; point to even address for word
	MOV	AX,[ESI]	; read word data from addrs 0002h
	CMP	AX,DX		; verify written word data
	JNZ	TST3ERR

	MOV	EDI,CPUST	; point to CPU status port
	TEST	BYTE PTR FS:[EDI], 11000000B ; is D6 or D7 = 1?
	JNE	TST2ERR 	; yes, all zero; no parrity error

	MOV	SI,BOUNDRYA	; reset offset
	ADD	ESI,10000H	; point to next 64KB bank
	CMP	ESI,EBP 	; last DRAM address is in EBP
	JL	TST3LOP3	; otherwise go on next test

;-------------------------------------------------------------------------+
;[Test 4] DRAM Address Line Test					  !
;									  !
; Purpose: To verify DRAM address lines and check for shorts and opens on !
;	   the DRAM address bus and internal RAM chip addressing.	  !
;									  !
; Description:								  !
;	   Shorted Address Line Test for each bank:			  !
;	      A 32-bit data pattern, 'FFFFFFFF'h, is written to the 04h   !
;	      base address of SRAM bank. For the next succeeding address  !
;	      line, a 32-bit pattern of '00000000'h is written (NOT verif-!
;	      ied), scanning the base  address for data integrity FFFFFFFF!
;	      h and NO parity errors. The testing proceeds with locations !
;	      defined by each succeeding address line (until END of DRAM) !
;	      written with the '00000000'h pattern, each time scanning the!
;	      base  address for the 'FFFFFFFF'h pattern and verifying NO  !
;	      parity errors occurred. In this manner each succeeding addr-!
;	      ess line is verified to NOT conflict with the address under !
;	      test. The base   address is moved to the next succeeding	  !
;	      address line, and the 'FFFFFFFF'h data pattern is written to!
;	      this address. The algorithm proceeds repeatly until the add-!
;	      ress under test reaches END of DRAM.			  !
;									  !
; BEGIN 								  !
;     FOR current bank = DRAM						  !
;	 FOR base    address = 04h, 08h, 10h, 20h, 40h, 80h, 100h, 200h,  !
;			       80h, ......., 4000h, 8000h		  !
;	     current pattern = FFFFFFFFh				  !
;	     reverse pattern = 00000000h				  !
;	     write current pattern to current address			  !
;									  !
;	    FOR succeeding address = 08h, 10h, 20h, 40h, ....., 8000h	  !
;		write reverse pattern to succeeding address		  !
;		read/verify current address		   ERROR 2C	  !
;		check for memory no parity error	   ERROR 2D	  !
;	    NEXT succeeding address					  !
;									  !
;	 NEXT base address						  !
;     NEXT bank 							  !
;									  !
;     base address = 0000:0000h 					  !
;     write 'FFFFFFFF'h to base address 				  !
;     FOR current bank = DRAM						  !
;	 FOR current address = 04h, 08h, 10h, 20h, ......, 8000h	  !
;	    FOR current pattern = 00000000h, 01010101h			  !
;		write current pattern to current address		  !
;		read/verify base address		   ERROR 2E	  !
;		check for memory no parity error	   ERROR 2F	  !
;	    NEXT pattern						  !
;	 NEXT address							  !
;     NEXT bank 							  !
; END									  !
;									  !
; ERRORS:								  !
; 2C Address Bus short							  !
;	  EAX= read data    EBX= reverse pattern   DS:SI= current address !
;			    ES:[BP]= succeeding address 		  !
;	  17th byte of RTC user RAM = error code			  !
;									  !
; 2E Address Bus Open							  !
;	  EAX= read data    EBX= expected data	   DS:SI= current address !
;	  17th byte of RTC user RAM = error code			  !
;									  !
; 2D & 2F Memory error, may be parity or circuit			  !
;	  AX= 4, SRAM parity err		   DS:SI= base address	  !
;	  17th byte of RTC user RAM = error code			  !
;-------------------------------------------------------------------------+
TST4:
	CALL	DELAYSEC
	MOV	ESI,OFFSET TST0NAME
	MOV	ECX,TST0NAMEND-TST0NAME ;count bytes
	CALL	DISPLAYMORE	; point cursor to test name
	MOV	ESI,OFFSET TST4NAME
	MOV	ECX,35		; max 35 bytes in string
	CALL	DISPLAYMORE	; display test 4 name

	MOV	EDX,0		; we will put last address here
	MOV	EAX,0E00H	; read saved MB size in RTCRAM
	CALL	READRTC 	; get last address in AL
	MOV	DL,AL
	SHL	EDX,20		; convert size to flat address

	MOV	ECX,00004H	; offset of first base address

TST4LOP1:
	PUSH	ECX		; save it as next base address offset
	MOV	EBX,0FFFFFFFFH	; get current pattern
	MOV	ESI,BOUNDRY	; set first base address in SRAM
	ADD	ESI,ECX 	; plus offset = current base address
	MOV	[ESI],EBX	; write 'FFFFFFFF' to base address

TST4LOP2:
	SHL	ECX,1		; get succeeding address offset
	CMP	ECX,EDX 	; do we reach the last address?
	JG	TST4LOP3	; go and write next succeeding address

	MOV	EBP,BOUNDRY	; setup next succeeding address
	ADD	EBP,ECX 	; plus offset = current succeeding address
	MOV	DWORD PTR [EBP],0   ; write reverse data to succeeding address
	MOV	EAX,[ESI]	; read current data from current address
	CMP	EAX,EBX 	; verify data
	JNE	TST4_ERR	; jump if bad (no jump is fast)

	MOV	EDI,CPUST	; point to CPU status port
	TEST	BYTE PTR FS:[EDI],0C0H ; keep D7&D6, mask out other bits
	JZ	TST4LOP2	; if zero, we have no parrity error
	JMP	TST2ERR 	; if not zero, we have parrity error

TST4LOP3:
	POP	ECX		; get last base address offset back
TST4LOP4:
	SHL	ECX,1		; prepare next base address offset
	CMP	ECX,EDX 	; do we reach the last address?
	JL	TST4LOP1	; go and write next base address

TST4LOP5:
	MOV	ESI,BOUNDRY	; it is the only base address for open test
	MOV	EAX,0FFFFFFFFH
	MOV	[ESI],EAX	; write data to it
	MOV	ECX,00002H	; offset of succeeding address

TST4LOP6:
	SHL	ECX,1		; get succeeding address offset
	CMP	ECX,EDX 	; do we reach the last address?
	JG	TST4LOP7	; go and write next succeeding address

	MOV	EBP,BOUNDRY	; setup next succeeding address
	ADD	EBP,ECX 	; plus offset = current succeeding address
	MOV	DWORD PTR [EBP],0   ; write reverse data to succeeding address
	MOV	EAX,[ESI]	; read current data from current address
	CMP	EAX,EBX 	; verify data
	JNE	TST4_ERR	; jump if bad (no jump is fast)

	MOV	EDI,CPUST	; point to CPU status port
	TEST	BYTE PTR FS:[EDI],0C0H ; keep D7&D6, mask out other bits
	JZ	TST4LOP6	; if zero, we have no parrity error
	JMP	TST2ERR 	; if not zero, we have parrity error

TST4_ERR:
	JMP	TST1ERR 	; show msg and wait reset key
TST4LOP7:
				; end of address bus open/short test
;-------------------------------------------------------------------------+
;[Test 5]  DRAM Data Fast Exchange Function TEST			  !
;									  !
;Purpose:  To test ability of all DRAM memory cells to hold a zero or a   !
;	   one and detec dynamic data/address faults.			  !
;									  !
;Description: The test is very fast because it takes advantage of various !
;	   CPU instructions. Three passes from the first to last memory   !
;	   address of DRAM are required to verify address and data faults !
;	   on both the data and parity DRAM.				  !
;									  !
; BEGIN 								  !
;     FOR DRAM first to last by 4's Do					  !
;	  write address = "FFFFFFFF"					  !
;     END (for DRAM)							  !
;									  !
;     FOR DRAM first to last by 4's Do					  !
;	  exchange "FFFFFFFF" with "00000000"				  !
;	  verify read data = "FFFFFFFF" 		   ERROR 31	  !
;	  read pattern "00000000"					  !
;	  verify read data = "00000000" 		   ERROR 31	  !
;									  !
;	  exchange "00000000" with "F4F4F4F4"				  !
;	  verify read data = "00000000" 		   ERROR 31	  !
;	  read pattern "F4F4F4F4F"					  !
;	  verify read data = "F4F4F4F4" 		   ERROR 31	  !
;     END (for DRAM)							  !
;									  !
;     Verify parity error				   ERROR 32	  !
; END									  !
;									  !
; ERRORS:								  !
; 31  Address Bus or Memory Data error					  !
;     EAX= read data	EBX= new write data	DS:SI= current or next	  !
;			EBP= last written data	       address		  !
;     17th byte of RTC user RAM = error code				  !
;									  !
; 32  Memory error, may be parity or circuit				  !
;     AX= 8, DRAM parity err						  !
;     17th byte of RTC user RAM = error code				  !
;-------------------------------------------------------------------------+
TST5:
	CALL	DELAYSEC
	MOV	ESI,OFFSET TST0NAME
	MOV	ECX,TST0NAMEND-TST0NAME ;count bytes
	CALL	DISPLAYMORE	; point cursor to test name
	MOV	ESI,OFFSET TST5NAME
	MOV	ECX,SHOWSIZEND-TST5NAME ; include message 'SRAM 0 KB DRAM 0 KB'
	CALL	DISPLAYMORE	; display test 5 name

	CALL	LASTADDR	; upon return, EDX is last DRAM address
	MOV	EDI,FIRSTRAM	; get 1st address of unused memory

	MOV	EAX,0FFFFFFFFH	; set flood data pattern
	MOV	ECX,EDX 	;
	SUB	ECX,EDI 	; the bytes need test
	SHR	ECX,2		; divid bytes by 4, convert to dword size
	CLD			; clear dir flag so string instr incrmnt DI
	REP	STOSD		; flood segment
				; end first scan

	MOV	EBX,0FFFFFFFFH	; set last written pattern
	MOV	EBP,00000000H	; set revers pattern for write
	MOV	ESI,FIRSTRAM	; start from 1st available byte

TST5LOP1:
	MOV	ECX,BOUNDRY
	SUB	ECX,ESI
	SHR	ECX,2		; set loop count as dword numbers
TST5LOP2:
	MOV	EAX,EBP 	; get revers pattern
	XCHG	[ESI],EAX	; read 'FFFFFFFF',write '00000000'
	CMP	EAX,EBX 	; verify data
	JNE	TST5ERR 	; jump if bad (no jump is fast)

	LODSD			; read '00000000' back and inc ESI by 4
	CMP	EAX,EBP
	LOOPZ	TST5LOP2	; loop as long as ECX not 0 and no error
	JNE	TST5ERRA	; jump if bad (no jump is fast)

	MOV	EDI,CPUST	; point to CPU status port
	TEST	BYTE PTR FS:[EDI], 0C0H ; keep D7&D6, mask out other bits
	JZ	TST5LOP2A	; if zero, we have no parrity error
	JMP	TST2ERR 	; if not zero, we have parrity error

TST5LOP2A:
	CALL	UPDATESIZE
	MOV	ECX,10000h/4	; dword counter
	CMP	ESI,EDX 	; do we reach last address?
	JL	TST5LOP2	; no,go on r/w/r test
				; end second scan

	MOV	EBX,0		; set last written pattern
	MOV	EBP,0F4F4F4F4H	; after test,all address have F4
	MOV	ESI,FIRSTRAM	; start from 1st available byte
	MOV	ECX,BOUNDRY
	SUB	ECX,ESI
	SHR	ECX,2		; set loop count as dword numbers

TST5LOP3:
	MOV	EAX,EBP 	; get revers pattern
	XCHG	[ESI],EAX	; read '0000',write 'F4F4F4F4'
	CMP	EAX,EBX 	; verify data
	JNE	TST5ERR 	; jump if bad (no jump is fast)

	LODSD			; read 'F4F4' back and inc ESI by 4
	CMP	EAX,EBP
	LOOPZ	TST5LOP3	; loop as long as ECX not 0 and no error
	JNE	TST5ERRA	; jump if bad (no jump is fast)        !

	MOV	EDI,CPUST	; point to CPU status port
	TEST	BYTE PTR FS:[EDI], 0C0H ; keep D7&D6, mask out other bits
	JZ	TST5LOP3A	; if zero, we have no parrity error
	JMP	TST2ERR 	; if not zero, we have parrity error

TST5LOP3A:
	CALL	UPDATESIZE
	MOV	ECX,10000h/4	; dword counter
	CMP	ESI,EDX 	; do we reach last address?
	JL	TST5LOP3	; no,go on r/w/r test
				; end third scan

	MOV	AX,2000h	; RTC 20h contains loopcount low byte
	CALL	READRTC
	MOV	BL,AL		; save loopcount low byte
	MOV	AX,2100h	; RTC 21h contains loopcount hi byte
	CALL	READRTC
	MOV	BH,AL		; save loopcount hi byte

	INC	BX		; increase loop count
	MOV	AH,20h		; RTC RAM 20h contains loopcount low byte
	MOV	AL,BL		; update loopcount
	CALL	WRITERTC
	MOV	AH,21h		; RTC RAM 21h contains loopcount hi byte
	MOV	AL,BH		; update loopcount
	CALL	WRITERTC

	MOV	ESI,OFFSET SHOWCOUNT
	MOV	ECX,83		; count bytes
	CALL	DISPLAYMORE	; position cursor to least significant digit
	MOV	AX,BX		; put loopcount to AX for display
	CALL	UPDATENUM	; display loopcount at terminal srceen

	CALL	DELAYSEC
	CALL	DELAYSEC	; delay for user to see the msg on screen
	JMP	TST1		; go on loop

TST5ERRA:
	MOV	EBX,EBP 	; move expected data to EBX
TST5ERR:
	JMP	TST1ERR 	; show msg and wait reset key

;********* all these subroutins are vailid only in Protect Mode *******

;-------->>>>>------------------+
WAITRESET	PROC	NEAR	;
;-------------------------------+---------------------------------------+
;on entry: none 							!
;on exit : return to loader in EPROM upon reset, otherwise nomal return !
;regs destroied: none							!
;purpose : this proc can check the console reset key pressed or not	!
;-------------------------------+---------------------------------------+

	PUSH	EDX
	MOV	EDX,_ABS
	MOV	BYTE PTR FS:[EDX],01H	;enable KEYBOARD (device address 01)

	PUSH	ECX
	MOV	ECX,0FFH
	LOOP	$		; time delay
	POP	ECX

	MOV	EDX,CPUST
	TEST	WORD PTR FS:[EDX],100H	;does reset key pressed?
	JNZ	EXIT

	MOV	EDX,_ABS
	MOV	BYTE PTR FS:[EDX],05H	;enable CRT (device address 05)
	POP	EDX
	RET

EXIT:
	MOV	EDI,CPUFLG	; get i/o port
	MOV	DX,02H
	MOV	FS:[EDI],DX	; clear parity error condition

	CALL	LASTADDR	; upon return, EDX is last DRAM address
	MOV	EDI,FIRSTRAM	; get 1st address of unused memory

	MOV	EAX,0F4F4F4F4H	; set flood data pattern
	MOV	ECX,EDX 	;
	SUB	ECX,EDI 	; the bytes need write
	SHR	ECX,2		; divid bytes by 4, convert to dword size
	CLD			; clear dir flag so string instr incrmnt DI
	REP	STOSD		; flood segment

	MOV	EDX,_ABS
	MOV	BYTE PTR FS:[EDX],05H	;enable CRT (device address 05)
	POP	EDX
	POP	EDX		; balance the stack
				; end house keeping
	LGDT	TGDT		; restore the GDT
	JMP	PWORD PTR IJUMP ; jump to bootstrap on EPROM, exit from @DG

;-------------------------------+
WAITRESET	ENDP		;
;--------<<<<<------------------+
;-------->>>>>------------------+
DELAYSEC	PROC	NEAR	;
;-------------------------------+---------------------------------------+
;on entry: none 							!
;on exit : none 							!
;regs destroied: none							!
;-------------------------------+---------------------------------------+

	PUSH	ECX
	MOV	ECX,0FFFFFH
	LOOP	$		; time delay
	POP	ECX
	RET
;-------------------------------+
DELAYSEC	ENDP		;
;--------<<<<<------------------+
;-------->>>>>------------------+
LASTADDR	PROC	NEAR	;
;-------------------------------+---------------------------------------+
;on entry: none 							!
;on exit : EDX is the byte number of DRAM, or the last DRAM address + 1 !
;regs destroied: none							!
;-------------------------------+---------------------------------------+

	PUSH	EBP
	MOV	EBP,RTCLOCK	; point to RTC address select port
	MOV	EDX,0Eh 	; point to 1st byte on user RAM
	MOV	FS:[EBP],DL	;
	MOV	EBP,RTCLOC2	; point to RTC data r/w port
	MOV	DL,FS:[EBP]	; read DRAM size from RTC RAM
	SHL	EDX,20		; convert size to flat address

	MOV	EBP,BOUNDRY	; get ds base offset
	OR	EBP,30000H	; convert it to last SRAM address
	ADD	EDX,EBP 	; SRAM + DRAM = last memory address
	POP	EBP
	RET
;-------------------------------+
LASTADDR	ENDP		;
;--------<<<<<------------------+
;-------->>>>>------------------+
WRITERTC	PROC	NEAR	;
;-------------------------------+---------------------------------------+
;on entry: AH= RTC user RAM address (0E-FF),				!
;	   AL= data to be written to RTC user RAM			!
;on exit : none 							!
;regs destroied: none							!
;-------------------------------+---------------------------------------+
	PUSH	EBP		;
	MOV	EBP,RTCLOCK	; point to RTC address select port
	MOV	FS:[EBP],AH	; AH is the user ram address
	MOV	EBP,RTCLOC2	; point to RTC data r/w port
	MOV	FS:[EBP],AL	; write data code to RTC RAM
	POP	EBP		;
	RET			;
;-------------------------------+
WRITERTC	ENDP		;
;--------<<<<<------------------+---------------------------------------+
;The RTC user RAM address starts from 0Eh to FFh, total 50 bytes.	*
;Diagnostic reserved following address: 				*
;	[0Eh]= the DRAM memory size in unit of MB,bootstrap will read it*
;	[0Fh]= external counter for update DRAM size shown on screen	*
;	[1Eh]= error code store here during protect mode		*
;	[1Fh]= MNI flag; 0= no NMI, 1= NMI occured during diagnostics	*
;	[20h]= loop count low byte					*
;	[21h]= loop count hi byte					*
;-------->>>>>------------------+---------------------------------------+
READRTC 	PROC	NEAR	;
;-------------------------------+---------------------------------------+
;on entry: AH is the address of RTC user RAM,				!
;on exit : AL is the data read from RTC user RAM at address 'AH'	!
;regs destroied: none							!
;-------------------------------+---------------------------------------+
	PUSH	EBP		;
	MOV	EBP,RTCLOCK	; point to RTC address select port
	MOV	FS:[EBP],AH	; AH is the user ram address
	MOV	EBP,RTCLOC2	; point to RTC data r/w port
	MOV	AL,FS:[EBP]	; read data code from RTC RAM
	POP	EBP		;
	RET			;
;-------------------------------+
READRTC 	ENDP		;
;--------<<<<<------------------+
;-------->>>>>------------------+
DISPLAYMORE	PROC	NEAR	;
;-------------------------------+---------------------------------------+
;on entry: ESI is point to the first byte of string			;
;	   ECX is the number of bytes in the string			;
;on exit : none 							;
;reg destroyed: none							;
;-----------------------------------------------------------------------+
	PUSH	EAX
DISPLAYMLOP:
	MOV	AL,[ESI]	;
	CALL	DISPLAYONE
	INC	ESI
	LOOP	DISPLAYMLOP
	POP	EAX
	RET
;-------------------------------+
DISPLAYMORE	ENDP		;
;--------<<<<<------------------+
;-------->>>>>------------------+
DISPLAYONE	PROC	NEAR	;
;-------------------------------+---------------------------------------+
;on entry: AL is the ASCII code to be displayed 			;
;on exit : none 							;
;reg destroyed: none							;
;-----------------------------------------------------------------------+
	CALL	WAITRESET	; does reset key pressed? if, yes, end diagnos
	CALL	CHK5USPULS	; delay 5useconds before next character
	CALL	CHKIORDY	; be sure CRT is ready for next send
	PUSH	EDI
	MOV	EDI,_OBS	; point to output data bus strobe
	MOV	FS:[EDI],AL	; send out ascii
	POP	EDI
	RET
;-------------------------------+
DISPLAYONE	ENDP		;
;--------<<<<<------------------+
;-------->>>>>------------------+
ENABLE		PROC	NEAR	;
;-------------------------------+---------------------------------------+
;on entry: AL is the device address to be enabled, CRT=05, PRT= 	;
;on exit : none 							;
;reg destroyed: none							;
;-----------------------------------------------------------------------+
	PUSH	EDI
	MOV	EDI,_ABS	;point to I/O address bus strobe
	MOV	FS:[EDI],AL
	CALL	CHK5USPULS	;wait for two 5usec pulses
	CALL	CHKIORDY	;make sure I/O bus is ready
	POP	EDI
	RET
;-------------------------------+
ENABLE		ENDP		;
;--------<<<<<------------------+
;-------->>>>>------------------+
CHK5USPULS	PROC	NEAR	;
;-------------------------------+---------------------------------------+
;purpose: check CPUST D0= 1 twice in order to get min dealy 5usec	;
;on entry: none 							;
;on exit : none 							;
;regs destroy: none							;
;-----------------------------------------------------------------------+
	PUSH	EDI
	PUSH	EAX
	MOV	EDI,CPUST	;point tp CPU I/O status port
CHK5USPULSA:
	MOV	AL,BYTE PTR FS:[EDI]
	TEST	AL,01H		;D0=1, 5usec pulse for ABS/, OBS/, CBS/
	JZ	CHK5USPULSA	;try again, be sure got first pulse
CHK5USPULSB:
	MOV	AL,BYTE PTR FS:[EDI]
	TEST	AL,01H		;D0=1, 5usec pulse for ABS/, OBS/, CBS/
	JZ	CHK5USPULSB	;try again in order to get second pulse
	POP	EAX
	POP	EDI
	RET
;-------------------------------+
CHK5USPULS	ENDP		;
;--------<<<<<------------------+
;-------->>>>>------------------+
CHKIORDY	PROC	NEAR	;
;-------------------------------+---------------------------------------+
;purpose: check CPUST D3=1 as I/O ready 				;
;on entry: none 							;
;on exit : none 							;
;regs destroy: none							;
;-----------------------------------------------------------------------+
	PUSH	EDI		; watch your push-pop pair in this procedure
	MOV	EDI,CPUST
	TEST	BYTE PTR FS:[EDI],08H ;D3=0,I/O not ready;D3=1,I/O raedy
	POP	EDI		; this push-pop should inside next jump loop
	JZ	CHKIORDY	; if not ready, let's wait
	RET
;-------------------------------+
CHKIORDY	ENDP		;
;--------<<<<<------------------+
;-------->>>>>------------------+
UPDATENUM	PROC	NEAR	;
;-------------------------------+---------------------------------------+
;purpose: convert word hex number to decimal and display it.		!
;on entry: AX contains hex number, cursor points to least signif digit. !
;on exit : none 							!
;regs destroy: none							!
;-----------------------------------------------------------------------+
	PUSH	EAX
	PUSH	EBX
	PUSH	ECX
	PUSH	EDX

	MOV	EBX,10		; set divisor
	MOV	ECX,4		; maximun 5 digits, there are 4 times DIV
UPDATENUM1:
	MOV	EDX,0		; clear remainder
	DIV	BX		; (DX+AX)/BX=AX+DX
	ADD	DL,30h		; convert remainder to ascii
	XCHG	AX,DX		; put ascii to AX for display
	CALL	DISPLAYONE	; display this ascii to screen

	MOV	AL,08h		; ascii 08h is back space
	CALL	DISPLAYONE	; mov cursor back
	MOV	AL,08h		; ascii 08h is back space
	CALL	DISPLAYONE	; set cursor to next digit position

	CMP	EDX,0		; is quotient zero?
	JE	UPDATENUM2	; it is zero, no more display

	MOV	AX,DX		; restore quotien for next div
	LOOP	UPDATENUM1	; continue division

	ADD	AL,30h		; convert last quotient to ascii
	CALL	DISPLAYONE	; display this ascii to screen

UPDATENUM2:
	POP	EDX
	POP	ECX
	POP	EBX
	POP	EAX
	RET
;-------------------------------+
UPDATENUM	ENDP		;
;--------<<<<<------------------+
;-------->>>>>------------------+
UPDATESIZE	PROC	NEAR	;
;-------------------------------+---------------------------------------+
;purpose: show dram memory size on screen in  64KB increasment		;
;on entry:none on register, but cursor point to least significant digit ;
;on exit :none, but cursor point to least significsnt digit		;
;regs destroy: none							;
;-----------------------------------------------------------------------+
	PUSH	ESI
	PUSH	EDX
	PUSH	ECX
	PUSH	EAX

	MOV	EAX,ESI 	; save surrent address
	ADD	EAX,BASE	; add DS base = physical address
	MOV	ECX,1024	; divided by 1024
	CMP	EAX,40000H	; DRAM start from 40000h
	JG	UPDATESIZEA

	PUSH	ECX
	MOV	ESI,OFFSET SRAMSIZE
	MOV	ECX,DRAMSIZE-SRAMSIZE	;count bytes
	CALL	DISPLAYMORE	; point cursor to 'SRAM 0 KB'
	POP	ECX

	MOV	EDX,0		; clear remainder
	DIV	ECX		; (EDX+EAX)/ECX= EAX + EDX
	CALL	UPDATENUM	; convert size to decimal and display
	JMP	UPDATESIZEB

UPDATESIZEA:
	PUSH	ECX
	MOV	ESI,OFFSET DRAMSIZE
	MOV	ECX,DRAMSIZEND-DRAMSIZE ;count bytes
	CALL	DISPLAYMORE	; point cursor to 'DRAM 0 KB'
	POP	ECX

	SUB	EAX,40000H	; deduct sram size
	MOV	EDX,0		; clear remainder
	DIV	ECX		; (EDX+EAX)/ECX= EAX + EDX
	CALL	UPDATENUM	; convert size to decimal and display

UPDATESIZEB:
	POP	EAX
	POP	ECX
	POP	EDX
	POP	ESI
	RET
;-------------------------------+
UPDATESIZE	ENDP		;
;--------<<<<<------------------+

TGDT	LABEL	PWORD
	DW	0030h
	DD	0h

IJUMP	LABEL	PWORD
	DD	0FFE9h
	DW	034h

COPYRIGHT	DB	03H, 06H, 0AH, 22 DUP(20H), 0EH, '****', 20H
		DB	'CS-386 CUSTOMER DIAGNOSTIC ****', 0FH, 0AH, 0AH, 0DH
		DB	20 DUP(20H),'Copyright, Wang Laboratories, Inc., 1989'
REVISION	DB	8 DUP(20H),'Rev 1960', 0DH, 0AH, 0AH, 0AH, 0AH
		DB	9 DUP(20H),'TEST NAME:', 39 DUP(20H),'LOOP COUNT:'
		DB	5 DUP(20H), 30H, 0DH, 0AH, 0AH, 10 DUP(20H),'MESSAGES:'
		DB	0DH, 12 dup(0AH), 20 DUP(20H), 'Press ', 0Eh
		DB	'SHIFT+RESET',0Fh,' key to terminate Diagnostics'
AFTER_COPYRIGHT:
SHOWCOUNT	DB	01H, 7 DUP(0AH), 69 DUP(09H), 5 DUP(20H), 0EH
TST0NAME	DB	01H, 12 DUP(0AH), 80 DUP(20H), 0DH, 0AH, 80 DUP(20H)
		DB	0DH, 0AH, 80 DUP(20H), 0DH, 0AH, 80 DUP(20H)
		DB	0DH, 0AH, 80 DUP(20H), 0DH, 0AH, 80 DUP(20H)
		DB	01H, 7 DUP(0AH), 20 DUP(09H), 0FH
TST0NAMEND:
TST1NAME	DB	'CPU Memory Data Bus Test', 11 DUP(20H)
TST2NAME	DB	'RAM Parity Generator/Check Test', 4 DUP(20H)
TST3NAME	DB	'RAM Byte Decode Address Line Test', 20H, 20H, 20H
TST4NAME	DB	'CPU Memory Address Bus Test', 8 DUP(20H)
TST5NAME	DB	'Memory Data Fast Exchange Test', 5 DUP(20H)
SHOWSIZE	DB	0DH, 5 DUP(0AH), 30 DUP (20H),'SRAM', 5 DUP(20H)
		DB	30H, 20H, 'KB', 0Ah, 0AH, 13 DUP(08H), 'DRAM'
		DB	5 DUP(20H), 30H, 20H, 'KB'
SHOWSIZEND:
SRAMSIZE	DB	01H, 12 DUP(0AH), 34 DUP(09H), 5 DUP(20H)
DRAMSIZE	DB	01H, 14 DUP(0AH), 34 DUP(09H), 5 DUP(20H)
DRAMSIZEND:
ERRMSG		DB	01H, 10 DUP(0AH), 21 DUP(20H), 0EH, 'Error was found:'
		DB	0FH, 0DH, 0AH, 0AH, 25 DUP(20H), 'at address ='
ADDRESS 	DB	20H, 8 DUP(30H), 20 DUP(20H)
		DB	0DH,0AH,70 DUP(20H),0DH,0AH,26 DUP(20H),'read data ='
READDATA	DB	20H, 8 DUP(30H), 20 DUP(20H)
		DB	0DH,0AH,70 DUP(20H),0DH,0AH,22 DUP(20H)
		DB	'expected data ='
EXPECDATA	DB	20H, 8 DUP(30H), 20 DUP(20H)
		DB	0DH, 0AH, 70 DUP(20H), 0DH, 0AH, 21 DUP(20H)
ERRMSGEND:
TST2MSG 	DB	'Unexpect parity error occurred'
TST2MSGA	DB	'Force parity bit generation failure'
TST2MSGB	DB	'Can not clear parity error'
TST2MSGEND:
ASCIIFONT	DB	30H, 31H, 32H, 33H, 34H, 35H, 36H, 37H, 38H, 39H
		DB	41H, 42H, 43H, 44H, 45H, 46H
SHIFTKEY	DB	01H, 21 DUP(0AH), 26 DUP(09H), 'SHIFT+RESET'
RESETKEY	DB	01H, 21 DUP(0AH), 26 DUP(09H), 0EH, 'SHIFT+RESET', 0FH
RESETKEYEND:
FIRSTRAM	DD	0H
BASE		DD	0H
BOUNDRY 	DD	0H
BOUNDRYA	DW	0H

STACK_BUFFER	LABEL	PWORD
CODE	ENDS
	END
