$title (' 80386 initialization  ---  PVAM mode')
name	reset
;
;********************************************************************
; Upon reset the 80386 starts executing at address 0fffffff0h. The
; upper 12 address bits remain high until a far call or jmp is executed
;
; Assume the following:
;
;  1. a short jump at address 0fffffff0h causes execution to begin at
;     START in the segment RESET_CODE.
;  2. segment RESET_CODE is based at physical address 0ffffc000h,
;     i.e. at the start of the last 16k in the memory.
;     you must adjust the program's location .
;**********************************************************************
;
;	 TOTAL MEMORY (0 - 8M) ALLOCATION MAP
;**********************************************************************
;
; ADDRESS
;
;    0 H |------------------------------------------|
;	 |   TEMP_GDT (for bstrap copy TSS,LDT from ROM to RAM) reserve for $init
;   40 H |------------------------------------------|
;	 |   GDT ( loader, sytem use)		  9 |
;   A0 H |------------------------------------------|
;	 |   LDT ( system)			  7 |
;   E0 H |------------------------------------------|
;	 |   TSS (system)			  67|
;  160 H |------------------------------------------|
;	 |					    |
;	 .					    .
;	 .   SYSTEM LOADED HERE(TOTAL 192K)	    .
;	 .					    .
;
; 50160H |------------------------------------------|
;	 .					    .
;	 .   SYSTEM RUNNING SPACE(begin)	    .
;	 .					    .
;
; 51000H |------------------------------------------| ---
;	 |   TSS (loader's TSS		  )	    |	 |
; 51080H |------------------------------------------|	 |
;	 |   LDT (loader's LDT		  )	    |	 |
; 510e0H |------------------------------------------|	 |
;	 |   DATA (loader's writable data ) + I/O BUFFER |  when RUNNING the system
; 51600H |------------------------------------------|	 |  this area will be OVERWRTIE
;	 |   STACK (loader's stack)		    |	 |  no use anymore
; 51700H |------------------------------------------|	 |
;	 |   TEMP AREA (loader builds system use)   |	 |
; 51780H |------------------------------------------| ----
;	 .					    .
;	 .					    .
;   2M H .   SYSTEM RUNNING SPACE(end)		    .
;	 .					    .
;	 .					    .
;   8M H |__________________________________________|
;
;************************************************************
;
;define the adjustment factor
adjust1 	equ 0ec00h
adjust2 	equ 0fff0h
adjust3 	equ 0ffe9h
bootid		equ 0ffffffe0h
;define addresses to locate GDT  in RAM
;
gdtbase 	equ 00000040h
;
; note: initial task's ldt will be put 00051080h
;		       tss will be put 00051000h
;		       writable data will be put 000510e0h
;		       writable stack will be put  051700h (expand down)

;*******************  MAO BEGIN **************************
;-------------------------------------------------------------------------+
;									  !
; (c) Copyright, Wang Laboratories, Inc., 1988, 1989			  !
;     All rights reserved.						  !
;									  !
; PROGRAM NAME: WANG 2200 CS386 Power up diagnostics			  !
;									  !
; FILE NAME:	CS.SRC							  !
;									  !
; SUPPORT GROUP: Engineering Service Department 			  !
;		 Wang Computer (Taiwan) LTD.				  !
;		 2, Science Road One	   ZIP 20077			  !
;		 Hsinchu, Taiwan, Republic of China			  !
;									  !
; REVISION HISTORY:							  !
; DATE	       PROGRAMMER      DESCRIPTION				  !
;  1-16-1989   C.C. MAO        R5930  for PCA can not disable NMI	  !
;  5-30-1989   C.C. Mao        R5950  for PCA R0M0 onward		  !
; TEST ROUTINE: 							  !
;EPROM MEMORY LOCATION FROM FFFFC000H TO FFFFFFFFH			  !
;POWER RESET LOCATION FFFFFFF0H 					  !
; NOTE: Test 0 is indicated by turing the CPU LED "on". 		  !
;[Test 1]  SRAM Data Bus Test						  !
; Purpose: To verify that no shorts or opens exist on SRAM data bus.	  !
;	   Some faults with the SRAM parity logic will be detected.	  !
;									  !
;[Test 2]  SRAM Parity Generator/Check Test				  !
; Purpose: To verify SRAM parity error circuity.			  !
;									  !
;[Test 3]  SRAM Byte Decode Address Line Test				  !
; Purpose: To check for shorts and opens on the SRAM byte decode address  !
;	   lines.							  !
;									  !
;[Test 4]  SRAM Address Line Test					  !
; Purpose: To verify SRAM address lines and check for shorts and opens on !
;	   the SRAM address bus and internal RAM chip addressing.	  !
;									  !
;[Test 5]  SRAM Data Fast Exchange Function TEST			  !
;Purpose:  To test ability of all SRAM memory cells to hold a zero or a   !
;	   one and detec dynamic data/address faults.			  !
;									  !
;[Test 6]  DRAM Data Bus Test						  !
; Purpose: To verify that no shorts or opens exist on DRAM data bus.	  !
;	   Some faults with the DRAM parity logic will be detected.	  !
;	   It also stores memory size at 1st byte of RTC user RAM.	  !
;									  !
;[Test 7]  DRAM Parity Generator/Check Test				  !
; Purpose: To verify DRAM parity error circuity.			  !
;									  !
;[Test 8]  DRAM Byte Decode Address Line Test				  !
; Purpose: To check for shorts and opens on the DRAM byte decode address  !
;	   lines.							  !
;									  !
;[Test 9]  DRAM Address Line Test					  !
; Purpose: To verify DRAM address lines and check for shorts and opens on !
;	   the DRAM address bus and internal RAM chip addressing.	  !
;									  !
;[Test A]  DRAM Data Fast Exchange Function TEST			  !
;Purpose:  To test ability of all DRAM memory cells to hold a zero or a   !
;	   one and detec dynamic data/address faults.			  !
;									  !
;									  !
;-------------------------------------------------------------------------+

TIMESET EQU	     8000H	;120NS FOR EACH UNIT

CPUST	EQU	     8002H	;READ CPU/IO STATUS
				;D0= 1, 5us pulse of ABS/, OBS/, CBS/
				;D1= 1, CPU BUSY
				;D2= 1, KEY BOARD FUNCTION
				;D3= 1, I/O READY
				;D4= 1, TIME OUTME OUT
				;D5= 1, CPU HALT
				;D6= 1, SRAM PARITY ERROR
				;D7= 1, DRAM PARITY ERROR
				;D8= 1, primary reset in from 2200 bus
				;D9= 1, nomal boot; =0, run-in
				;D10=1, same as D2, but read cause clear
				;D11	not defined
				;D12	not defined
				;D13=1, same as D5, but read cause clear

CPUFLG	EQU	     8004H	;WRITE IO PORT
				;D0= 1, generate even parity bit for diag
				;  = 0, clear force even parity condition
				;D1= 1, clear parity error if occured anytime
				;  = 0, not clear parity error, if occured
				;D2= 1, clear 387 error if occured
				;  = 0, not clear 387 error if occured
				;D3= 1, CPU LED on
				;  = 0, CPU LED off

CPURDY	EQU	    8006H	;release CPU BUSY

RTCLOCK EQU    0FFFF8008H	;realtime clock address strobe
RTCLOC2 EQU    0FFFF800AH	;realtime clock data strobe, data r/w port
				;1st byte RTC user rAM = memory size in MB
				;17byte user ram = error code in protect mode
				;18th byte = MNI flag; 0= no NMI, 1=NMI occured
				;19th byte = loop count low byte
				;20th byte = loop count hi byte

IBGET	EQU	     800CH	;get input data from bus
_ABS	EQU	     8012H	;address bus strobe
_OBS	EQU	0FFFF8014H     ;output data to bus
CBS	EQU	     8018H	;command data bus strobe
;************  MAO END ***********************
;
dummy	segment rw
	dw	0
dummy	ends
;
;*****************************************************************
;
;  RESET_CODE must be USE16 because the 386 initially executes in real mode
;
;*******************************************************************
;
;
reset_code	segment er public use16
assume ds:nothing, es:nothing
;
; 386 descriptor template
;
desc	struc
	lim_0_15	dw 0
	bas_0_15	dw 0
	bas_16_23	db 0
	access		db 0
	gran		db 0
	bas_24_31	db 0
desc	ends
;
; the following is the layout of the real GDT created by BLD386.
; we use the  .bld file to make the following layout.
; it is located in EPROM and will be copied to RAM.
;
; GDT[0].... null
; GDT[1].... gdt alias
; GDT[2].... idt alias
; GDT[3].... initial task tss ------base address point to rom for copy from
; GDT[4].... initial task tss alias-base address point to ram for copy to
; GDT[5].... initial task ldt ------base address ponit to rom for copy from
; GDT[6].... initial task ldt alias-base address ponit to ram for copy to
; ..........
; GDT[7]... reserve for loaded task
; GDT[8]... reserve for loaded task
;
; the descriptors in gdt total 9
;
gdt_entries	equ 9
;
gdt_alias	equ 1*size desc
idt_alias	equ 2*size desc
init_tss	equ 3*size desc
init_tss_a	equ 4*size desc
init_ldt	equ 5*size desc
init_ldt_a	equ 6*size desc
;
init_ldt_alias	equ 1*size desc
;
; access right byte for data and tss descriptor
;
ds_access	equ 010010010b
tss_access	equ 010001001b
;
;this temporary gdt will be used to set up the real gdt in ram
;
	org	adjust1
temp_gdt	label byte
null_des	desc <>
;use this descriptor we can access 4gb memory from physical address 0
flat_des	desc<0ffffh,0,0,92h,0cfh,0>
;the gdt address and limit in eprom
gdt_eprom	label pword
	dw	0047h
	dd	0ffffc0e0h
;the temporary gdt address and limit
tgdt_pword	label pword
	dw	end_temp_gdt -1
	dd	0
;the gdt address and limit in ram
gdt_pword      label pword
	dw	gdt_entries * size desc -1
	dd	gdtbase
end_temp_gdt	label byte
;
gdt_des_flat	equ ds:gdt_alias + gdtbase
;
init_tss_a_offset	equ ds:init_tss_a
init_tss_offset 	equ ds:init_tss
;
init_ldt_a_offset	equ ds:init_ldt_a
init_ldt_offset 	equ ds:init_ldt
;
; define pointer for first task switch
;
entry_pointer	label  pword
	dd 0
	dw ds:init_tss
;
;************* MAO BEGIN *****************
START:	CLI			; disable interrupts
	JMP	AFTER_COPYRIGHT

SRAMSIZE	DB	'SRAM', 20H, 20H, 20H,'256',20H,'KB',0DH,0AH,06H
ENDSRAMSIZE:
SRAMLOOP	DB	'SRAM', 20H, 20H, 20H,'256',20H,'KB', 45 DUP(20H)
		DB	'LOOP COUNT:', 5 DUP(20H), 30H, 0DH,0AH,06H
ENDSRAMLOOP:
DRAMSIZE	DB	'M', 5 DUP(20H),30H,20H,'KB', 08H, 08H, 08H, 08H
ENDDRAMSIZE:
LOOPCONT	DB	01h, 0Ah, 0Ah, 69 DUP(09H),6 DUP(20H)
ENDLOOPCONT:
DRAMCLEN	DB	0Ah, 0Dh, 15 DUP(20h), 0Dh
ENDDRAMCLEN:
DRAMLOOP	DB	0Dh, 5 DUP(09h),4 DUP(20H)
ENDDRAMLOOP:

COPYRIGHT	DB	'Copyright, Wang Laboratories, Inc., 1989'
		DB	20H, 20H, 20H, 20H, 20H
REVISION	DB	'Rev 5930', 0DH, 0AH, 0AH
		PUBLIC	COPYRIGHT

AFTER_COPYRIGHT:
	MOV	BX,CPUFLG	; select I/O port, direct memory mapping
	MOV	AX,08H		; D3= 1, cpu led on; D3= 0, cpu led off
	MOV	CS:[BX],AX	; turn CPU LED on,it offs at end Test 1
	XOR	ECX,ECX 	; set up counter for time delay

WITH_CPU_LED:
	AAM			; wait so CE can see LED is turned "on".
	LOOP	WITH_CPU_LED	; 83 clk x .125us/clk x 65536 = .679 secnd

				; disable NMI
				; disable 80x87 coprocessor interrupts

;-------------------------------------------------------------------------+
;[Test 1]  SRAM Data Bus Test						  !
;									  !
; Purpose: To verify that no shorts or opens exist on SRAM data bus.	  !
;	   Some faults with the SRAM parity logic will be detected.	  !
;									  !
; Description: Walking one and zero data patterns (32-bit) are written,   !
;	   then read/verified for correct data and NO parity error, on	  !
;	   bank basis. All patterns has an odd number of 'high'(1) bits.  !
;	   A double-bit rotating pattern, starting value '00000003'H, will!
;	   be written/read/verified for correct data and NO parity. These !
;	   patterns have an even number of 'high'(1) bits.		  !
;	   The routine uses address zero of SRAM of each 64KB bank, so	  !
;	   chip faults at that location will be flagged as well.	  !
;									  !
; BEGIN 								  !
;     FOR current bank = SRAM						  !
;	  current address = first in current bank			  !
;	 FOR current pattern = FFFFFFFF, 01, 02, 04, ...80000000, 0,	  !
;			       7FFFFFFF, BFFFFFFD, ......., FFFFFFFE	  !
;	     write current pattern to current address			  !
;	     read/verify current address		ERROR 01	  !
;	     check for memory parity error		ERROR 02	  !
;	 NEXT pattern							  !
;     NEXT bank 							  !
;									  !
;     FOR current bank = SRAM						  !
;	  current address = first in current bank			  !
;	 FOR current pattern = 03, 06, 0C, 18, 30,...., C0000000, 80000001!
;	     write current pattern to current address			  !
;	     read/verify current address		ERROR 03	  !
;	     check for memory parity error		ERROR 04	  !
;	 NEXT pattern							  !
;     NEXT bank 							  !
; END									  !
;									  !
; ERRORS:								  !
; 01 & 03 Data bus or memory data error 				  !
;	  EAX= read data    EBX= expected data	   DS:SI= current address !
;	  FS= error code						  !
;									  !
; 02 & 04 Memory error, may be parity or circuit			  !
;	  AX= 4, SRAM parity err					  !
;	  FS= error code						  !
;-------------------------------------------------------------------------+
TST1:

	MOV	AX,6		; D1=1 clear parity error,D2=1 clr 387
	MOV	CS:[BX],AX	; BX point to CPUFLG I/O port

	XOR	SI,SI		; start from address 0:0000
	MOV	DI,SI
TST1LOP1:
	MOV	AX,1		; setup error code
	MOV	FS,AX		; when error, FS is the error code

	MOV	AX,DI		; start from address 0:0000 to F000:0000
	MOV	DS,AX		;
	MOV	EBX,0FFFFFFFFH	; current data pattern
	MOV	EAX,01		; next pattern
	MOV	EDX,EAX 	; save next pattern
	MOV	CX,33		; 33 patterns from FFFFFFFF to 01h
	MOV	[SI],EBX	; write current pattern to current address

TST1LOP2:
	XCHG	[SI],EAX	; read data from currnt addr & write next
	CMP	EAX,EBX 	; verify read data against current pattern
	JNE	TST1_ERR	; jump if bad (no jump is fast)

	MOV	EBX,EDX 	; save current pattern
	SHL	EDX,1		; setup next data pattern
	MOV	EAX,EDX 	; ready to write data pattern from EAX reg
	DEC	CX
	JNZ	TST1LOP2       ; loop for next pattern test

	MOV	EAX,7FFFFFFFH	; next data pattern 7FFFFFFF
	MOV	EDX,EAX 	; prepare for shift right
	XCHG	[SI],EAX	; read data from currnt addr & write next
	CMP	EAX,EBX 	; verify read data against current pattern
	JNE	TST1_ERR	; jump if bad (no jump is fast)

	MOV	EBX,EDX 	; save current pattern for compare later
	ROR	EDX,1		; setup next data pattern
	MOV	EAX,EDX
	MOV	CX,32		; 32 patterns from 7FFFFFFF to FFFFFFFE

TST1LOP3:
	XCHG	[SI],EAX	; read data from currnt addr & write next
	CMP	EAX,EBX 	; verify read data against current pattern
	JNE	TST1_ERR	; jump if bad (no jump is fast)

	MOV	EBX,EDX 	; save current pattern for compare later
	ROR	EDX,1		; setup next data pattern
	MOV	EAX,EDX
	DEC	CX
	JNZ	TST1LOP3       ; loop for next pattern test

	MOV	AX,2		; setup error code for parrity check
	MOV	FS,AX		; error code in FS = 2
	MOV	BX,CPUST	; point to CPU status port
	MOV	AX,CS:[BX]	; read CPU status
	TEST	AX,01000000B	; keep D6, mask out other bits
	JNZ	TST1_ERR	; if D6 not zero; parrity error ocurred

	ADD	DI,1000H	; point to next 64KB bank
	CMP	DI,4000H	; last SRAM is 3:FFFFh
	JL	TST1LOP1       ; otherwise go on next test

	XOR	SI,SI		; start from address 0:0000 for
	MOV	DI,SI		; even number bits patterns rotate
TST1LOP4:
	MOV	AX,3		; setup error code
	MOV	FS,AX		; when error, FS is the error code

	MOV	AX,DI		; start from address 0:0000 to F000:0000
	MOV	DS,AX		;
	MOV	EBX,03H 	; current data pattern
	MOV	EAX,06H 	; next pattern
	MOV	EDX,EAX 	; save next pattern
	MOV	CX,32		; 32 patterns from 03 to 80000001h
	MOV	[SI],EBX	; write current pattern to current address

TST1LOP5:
	XCHG	[SI],EAX	; read data from currnt addr & write next
	CMP	EAX,EBX 	; verify read data against current pattern
	JNE	TST1_ERR	; jump if bad (no jump is fast)

	MOV	EBX,EDX 	; save current pattern
	ROL	EDX,1		; setup next data pattern
	MOV	EAX,EDX 	; ready to write data pattern from EAX reg
	DEC	CX
	JNZ	TST1LOP5       ; loop for next pattern test

	MOV	AX,4		; setup error code for parrity check
	MOV	FS,AX		; error code in FS
	MOV	BX,CPUST	; point to CPU status port
	MOV	AX,CS:[BX]	; read CPU status
	TEST	AX,01000000B	; keep D6, mask out other bits
	JNZ	TST1_ERR	; if D6 not zero; parrity error ocurred

	ADD	DI,1000H	; point to next 64KB bank
	CMP	DI,4000H	; last SRAM is 3:FFFFh
	JL	TST1LOP4       ; otherwise go on next test

	MOV	BX,CPUFLG	; select I/O port, direct memory mapping
	MOV	AX,0		; D3= 1, cpu led on; D3= 0, cpu led off
	MOV	CS:[BX],AX	; turn CPU LED off as end of Test 1
	JMP	TST2

TST1_ERR:
	MOV	DI,CPUFLG	; select I/O port, direct memory mapping
	MOV	CX,08H		; D3= 1, cpu led on; D3= 0, cpu led off
	MOV	CS:[DI],CX	; turn CPU LED on as an error ocurred

	MOV	DI,CPUST	; point to CPU status I/O port
	MOV	BP,CS:[DI]	; read	CPU sataus
	TEST	BP,200h 	; D9=0 loop on BIT is requested
	JZ	TST1

	JMP	$		; hang here if error

;-------------------------------------------------------------------------+
;[Test 2] SRAM Parity Generator/Check Test				  !
;									  !
; Purpose: To verify SRAM parity error circuity.			  !
;									  !
; WARNING: The NMI can not be disable, so watch your NMI service routine. !
;									  !
; Description: The test first writes a 32-bit pattern with an odd number  !
;	   of bits 'high'(1) in each byte and normal parity generation to !
;	   the lowest address in the bank. The data is then read/verified !
;	   and parity status checked for NO error. The FORCED parity fun- !
;	   ction is enabled which causes the EVEN parity logic to generate!
;	   a 'high'(1) on the four outputs, then read/verified again, with!
;	   a check to ensure parity error occurred. The FORCED parity fun-!
;	   ction is then disabled and the parity error is reset. The pat- !
;	   tern is then rotated and the algorithm is repeated within each !
;	   byte to test all the lines to the parity generator/checker	  !
;	   chips. Although 32-bit writes are performed in this test, each !
;	   byte is read back for test separately, starting from the least !
;	   significant byte and processing to the most significant byte   !
;	   of the 4-byte double-word.					  !
;									  !
; BEGIN 								  !
;     FOR current bank = SRAM						  !
;	  current address = first in current bank			  !
;	 FOR current pattern = 01010101h, 02020202h, 04040404h, .....	  !
;			       20202020h, 40404040h, 80808080h, 	  !
;	     write current pattern to current address			  !
;	     read/verify current address		ERROR 05	  !
;	     check for memory no parity error		ERROR 06	  !
;	     enable forced parity function				  !
;	     read/verify current address		ERROR 07	  !
;	     check for memory parity error occurred	ERROR 08	  !
;	     disable forced parity function				  !
;	 NEXT pattern							  !
;     NEXT bank 							  !
; END									  !
;									  !
; ERRORS:								  !
; 05 & 07 Data bus or memory data error 				  !
;	  EAX= read data    EBX= expected data	   DS:SI= current address !
;	  FS= error code						  !
;									  !
; 06 & 08 Memory error, may be parity or circuit			  !
;	  AX= 4, SRAM parity err					  !
;	  FS= error code						  !
;-------------------------------------------------------------------------+

TST2_ERR:
	MOV	DI,CPUFLG	; select I/O port, direct memory mapping
	MOV	CX,08H		; D3= 1, cpu led on; D3= 0, cpu led off
	MOV	CS:[DI],CX	; turn CPU LED on as an error ocurred

	MOV	DI,CPUST	; point to CPU status I/O port
	MOV	BP,CS:[DI]	; read	CPU sataus
	TEST	BP,200h 	; D9=0 loop on BIT is requested
	JZ	TST2

	JMP	$		; hang here if error

TST2:
;************** for old PCA without disable NMI ************************+
;;;;	JMP	TST3		; by pass parity check, temperary	*
;***********************************************************************+
	MOV	SI,0		; start from address 0:0000
	MOV	DS,SI		; start from address 0:0000
	MOV	BX,CPUST	; CS:[BX]= CPU status port
	MOV	DI,CPUFLG	; CS:[DI]= CPU flag I/O port
	MOV	EDX,01010101H	; current data pattern, odd '1'

TST2LOP1:
	MOV	SI,0		; reset SI to 0000h
	MOV	[SI],EDX	; write current pattern to current address

	MOV	AX,5		; setup error code
	MOV	FS,AX		; when error, FS is the error code
	MOV	AL,[SI] 	; read data from currnt addr first byte
	CMP	AL,DL		; verify read data against current pattern
	JNE	TST2_ERR	; jump if bad (no jump is fast)

	MOV	AX,6		; setup error code
	MOV	FS,AX		; when error, FS is the error code
	MOV	AX,CS:[BX]	; read CPU status
	TEST	AX,01000000B	; keep D6, mask out other bits
	JNZ	TST2_ERR	; if D6 not zero; parrity error ocurred

	INC	SI
	MOV	AL,[SI] 	; read data from second byte
	CMP	AL,DL		; verify read data against current pattern
	JNE	TST2_ERR	; jump if bad (no jump is fast)

	MOV	AX,6		; setup error code
	MOV	FS,AX		; when error, FS is the error code
	MOV	AX,CS:[BX]	; read CPU status
	TEST	AX,01000000B	; keep D6, mask out other bits
	JNZ	TST2_ERR	; if D6 not zero; parrity error ocurred

	INC	SI
	MOV	AL,[SI] 	; read data from third byte
	CMP	AL,DL		; verify read data against current pattern
	JNE	TST2_ERR	; jump if bad (no jump is fast)

	MOV	AX,6		; setup error code
	MOV	FS,AX		; when error, FS is the error code
	MOV	AX,CS:[BX]	; read CPU status
	TEST	AX,01000000B	; keep D6, mask out other bits
	JNZ	TST2_ERR	; if D6 not zero; parrity error ocurred

	INC	SI
	MOV	AL,[SI] 	; read data from fourth byte
	CMP	AL,DL		; verify read data against current pattern
	JNE	TST2_ERR	; jump if bad (no jump is fast)

	MOV	AX,6		; setup error code
	MOV	FS,AX		; when error, FS is the error code
	MOV	AX,CS:[BX]	; read CPU status
	TEST	AX,01000000B	; keep D6, mask out other bits
	JNZ	TST2_ERR	; if D6 not zero; parrity error ocurred

	MOV	CX,01H		; D0= 1, force parity generate error
	MOV	CS:[DI],CX	; we expect parity error at next read

	MOV	SI,0		; point to first address
	MOV	[SI],EDX	; write in order to get wrong parity
	MOV	AL,[SI] 	; read data from currnt addr first byte
	CMP	AL,DL		; verify read data against current pattern
	JNE	TST2_ERR	; jump if bad (no jump is fast)

	MOV	AX,CS:[BX]	; read CPU status
	TEST	AX,01000000B	; keep D6, mask out other bits
	JZ	TST2_ERR	; if D6 not zero; parrity error ocurred
	MOV	CX,02H		; D1= 1, clear parity error
	MOV	CS:[DI],CX	; expect parity error at next read
	MOV	AX,CS:[BX]	; read CPU status
	TEST	AX,01000000B	; keep D6, mask out other bits
	JNZ	TST2_ERR	; D6 not zero; parrity error can't reset
	MOV	CX,0		; D1= 0, not clear parity error
	MOV	CS:[DI],CX	; expect parity error at next read

	INC	SI
	MOV	AL,[SI] 	; read data from second byte
	CMP	AL,DL		; verify read data against current pattern
	JNE	TST2_ERR	; jump if bad (no jump is fast)

	MOV	AX,CS:[BX]	; read CPU status
	TEST	AX,01000000B	; keep D6, mask out other bits
	JZ	TST2_ERR	; if D6 not zero; parrity error ocurred
	MOV	CX,02H		; D1= 1, clear parity error; D0=0, normal
	MOV	CS:[DI],CX	; reset parity error at CPUSTU D6 bit
	MOV	AX,CS:[BX]	; read CPU status
	TEST	AX,01000000B	; keep D6, mask out other bits
	JNZ	TST2_ERR	; D6 not zero; parrity error can't reset
	MOV	CX,0		; D1= 0, not clear parity error
	MOV	CS:[DI],CX	; expect parity error at next read

	INC	SI
	MOV	AL,[SI] 	; read data from third byte
	CMP	AL,DL		; verify read data against current pattern
	JNE	TST2_ERR	; jump if bad (no jump is fast)

	MOV	AX,CS:[BX]	; read CPU status
	TEST	AX,01000000B	; keep D6, mask out other bits
	JZ	TST2_ERR	; if D6 not zero; parrity error ocurred
	MOV	CX,02H		; D1= 1, clear parity error; D0=0, normal
	MOV	CS:[DI],CX	; reset parity error at CPUSTU D6 bit
	MOV	AX,CS:[BX]	; read CPU status
	TEST	AX,01000000B	; keep D6, mask out other bits
	JNZ	TST2_ERR	; D6 not zero; parrity error can't reset
	MOV	CX,0		; D1= 0, not clear parity error
	MOV	CS:[DI],CX	; expect parity error at next read

	INC	SI
	MOV	AL,[SI] 	; read data from fourth byte
	CMP	AL,DL		; verify read data against current pattern
	JNE	TST2_ERR	; jump if bad (no jump is fast)
	MOV	AX,CS:[BX]	; read CPU status
	TEST	AX,01000000B	; keep D6, mask out other bits
	JZ	TST2_ERR	; if D6 not zero; parrity error ocurred
	MOV	CX,02H		; D1=1,clear parity err; D0=0,back to normal
	MOV	CS:[DI],CX	; reset parity error at CPUSTU D6 bit
	MOV	AX,CS:[BX]	; read CPU status
	TEST	AX,01000000B	; keep D6, mask out other bits
	JNZ	TST2_ERR	; D6 not zero; parrity error can't reset
	MOV	CX,0		; D1= 0, not clear parity error
	MOV	CS:[DI],CX	; expect parity error at next read

	MOV	CX,0		; D0= 0, normal parity generate
	MOV	CS:[DI],CX	; we expect parity no error at next read

	SHL	EDX,1		; setup next data pattern
	JNC	TST2LOP1	; if carry, do even '1' test
	JMP	TST3		; end of parity circult testing

;-------------------------------------------------------------------------+
;NMI Service Routine for power up stage diagnostics			  !
;									  !
; upon entry: no pre-requested paramenters				  !
; after exit: RTC user RAM 18th byte contain "01H" as NMI occured	  !
;	      (address 1Eh) = 01H					  !
; regesters destroyed: AX, EDI, 					  !
;									  !
;									  !
;-------------------------------+-----------------------------------------+
NMI_SRV 	Proc	far	;
;-------------------------------+
	NOP
	MOV	DI,CPUFLG	; select I/O port, direct memory mapping
	MOV	CX,08H		; D3= 1, cpu led on; D3= 0, cpu led off
	MOV	CS:[DI],CX	; turn CPU LED on as an error ocurred

	MOV	EDI,RTCLOCK	; point to RTC address select port
	MOV	AX,1F01H	; point to 18th byte on user RAM
	MOV	CS:[DI],AH	;
	MOV	EDI,RTCLOC2	; point to RTC data r/w port
	MOV	CS:[DI],AL	; write error code to RTC RAM
	NOP
	NOP
	IRET
;-------------------------------+
NMI_SRV 	endp		;
;-------------------------------+
;-------------------------------------------------------------------------+
;[Test 3] SRAM Byte Decode Address Line Test				  !
;									  !
; Purpose: To check for shorts and opens on the SRAM byte decode address  !
;	   lines.							  !
;									  !
; Description: Data is writen and verified to the beginning of each bank  !
;	   in byte and word patterns. Doubleword patterns have been tested!
;	   in the [Test 1] SRAM Data Line Test. The 80386 stores word data!
;	   in memory in reverse byte order i.e. the high-order byte follo-!
;	   wing the low-order byte. Therefore the high (most-significant) !
;	   byte of a data word is in the highest memory address. The word !
;	   read/write can be performed at both even and odd address.	  !
;									  !
; BEGIN 								  !
;     FOR current bank = SRAM						  !
;	  write 00h, 01h, 02h, 03h for the offset addresses 00h, 01h, 02h,!
;		03h respectively.					  !
;	  read/verify data 00h, 01h, 02h, 03h for the offset addresses	  !
;		00h, 01h, 02h, 03h respectively.	     ERROR 09	  !
;	  write 03h, 02h, 01h, 00h for the offset addresses 00h, 01h, 02h,!
;		03h respectively.					  !
;	  read/verify data 03h, 02h, 01h, 00h for the offset addresses	  !
;		00h, 01h, 02h, 03h respectively.	     ERROR 0A	  !
;	  write AA55h, CC33h for the offset address 00h, 02h respectively.!
;	  read/verify AA55h, CC33h for offset addresses 00h, 02h	  !
;		respectively.				     ERROR 0B	  !
;	  write 9966h for offset address 01h.				  !
;	  read/verify 6655h, 9966h, CC99h for offset addresses 00h, 01h,  !
;		02h respectively.			     ERROR 0C	  !
;     NEXT bank 							  !
; END									  !
;									  !
; ERRORS:								  !
; 09 & 0A,  byte decode address error					  !
;	  AL= read data,   BX & CX=  expected data,  DS:SI= current addres!
;	  FS= error code						  !
; 0B  word decode address error at even boundry 			  !
;	  AX= read data,   BX & CX=  expected data,  DS:SI= current addres!
;	  FS= error code						  !
; 0C  word decode address error at odd boundry				  !
;	  AX= read data,   BX, CX & DX=  expected data,  DS:SI= current   !
;	  FS= error code					address   !
;-------------------------------------------------------------------------+
TST3:
	MOV	BX,0100H	; byte patterns 00h, 01h, 02h, 03h
	MOV	CX,0302H	;

	XOR	DI,DI		; start from address 0:0000
TST3LOP1:
	MOV	AX,DI		; start from address 0:0000 to F000:0000
	MOV	DS,AX		;
	MOV	AX,09h		; setup srror code
	MOV	FS,AX
TST3LOP2:
	XOR	SI,SI		; start from address 0:0000
	MOV	[SI],BL 	; write byte 00h to address 0000h
	MOV	[SI+1],BH	; write byte 01h to address 0001h
	MOV	[SI+2],CL	; write byte 02h to address 0002h
	MOV	[SI+3],CH	; write byte 03h to address 0003h
	MOV	AL,[SI] 	; read byte data from addrs 0000h
	CMP	AL,BL		; verify written byte data
	JNZ	TST3_ERR
	INC	SI		; point to next byte address
	MOV	AL,[SI] 	; read byte data from addrs 0001h
	CMP	AL,BH		; verify written byte data
	JNZ	TST3_ERR
	INC	SI		; point to next byte address
	MOV	AL,[SI] 	; read byte data from addrs 0002h
	CMP	AL,CL		; verify written byte data
	JNZ	TST3_ERR
	INC	SI		; point to next byte address
	MOV	AL,[SI] 	; read byte data from addrs 0003h
	CMP	AL,CH		; verify written byte data
	JNZ	TST3_ERR

	MOV	AX,FS		; which byte patterns tested?
	CMP	AX,0AH		; are we first time here?
	JZ	TST30		; yes, jump to word patterns test
	MOV	BX,0203H	; byte patterns 03h, 02h, 01h, 00h
	MOV	CX,0001H	;
	MOV	AX,0Ah		; setup srror code
	MOV	FS,AX
	JMP	TST3LOP2

TST3_ERR:
	MOV	DI,CPUFLG	; select I/O port, direct memory mapping
	MOV	CX,08H		; D3= 1, cpu led on; D3= 0, cpu led off
	MOV	CS:[DI],CX	; turn CPU LED on as an error ocurred

	MOV	DI,CPUST	; point to CPU status I/O port
	MOV	BP,CS:[DI]	; read	CPU sataus
	TEST	BP,200h 	; D9=0 loop on BIT is requested
	JZ	TST3		; retry this test 3

	JMP	$		; hang here if error

TST30:
	ADD	DI,1000H	; point to next 64KB bank
	CMP	DI,4000H	; last SRAM is 3:FFFFh
	JL	TST3LOP1	; otherwise go on next test

	XOR	DI,DI		; word pattern test from 0:0000
TST3LOP3:
	MOV	AX,DI		; start from address 0:0000 to F000:0000
	MOV	DS,AX		;
	MOV	AX,0Ch		; setup srror code
	MOV	FS,AX
	MOV	BX,0AA55H	; word patterns 55, AA, 33, CC,
	MOV	CX,0CC33H	;
	XOR	SI,SI		; start from address 0:0000
	MOV	[SI],BX 	; write word 55, AA to addrs 00h
	MOV	[SI+2],CX	; write word 33, CC to addrs 02h
	MOV	AX,[SI] 	; read word data from addrs 0000h
	CMP	AX,BX		; verify written word data
	JNZ	TST3_ERR
	ADD	SI,2		; point to next word address
	MOV	AX,[SI] 	; read word data from addrs 0002h
	CMP	AX,CX		; verify written word data
	JNZ	TST3_ERR

	MOV	AX,0Bh		; setup srror code
	MOV	FS,AX
	MOV	BX,6655H	; new data in byte series 55, 66,
	MOV	CX,9966H	; 99, CC; in word serie are 6655,
	MOV	DX,0CC99H	; 9966 and CC99.


	XOR	SI,SI		; start from address 0:0000
	MOV	[SI+1],CX	; write word to address 0001h
	MOV	AX,[SI] 	; read word data from addrs 0000h
	CMP	AX,BX		; verify written word data
	JNZ	TST3_ERR
	INC	SI		; point to odd address for word
	MOV	AX,[SI] 	; read word data from addrs 0001h
	CMP	AX,CX		; verify written word data
	JNZ	TST3_ERR
	INC	SI		; point to even address for word
	MOV	AX,[SI] 	; read word data from addrs 0002h
	CMP	AX,DX		; verify written word data
	JNZ	TST3_ERR

	ADD	DI,1000H	; point to next 64KB bank
	CMP	DI,4000H	; last SRAM is 3:FFFFh
	JL	TST3LOP3	; otherwise go on next test

;-------------------------------------------------------------------------+
;[Test 4] SRAM Address Line Test					  !
;									  !
; Purpose: To verify SRAM address lines and check for shorts and opens on !
;	   the SRAM address bus and internal RAM chip addressing.	  !
;									  !
; Description:								  !
;	   Shorted Address Line Test for each bank:			  !
;	      A 32-bit data pattern, 'FFFFFFFF'h, is written to the 04h   !
;	      base address of SRAM bank. For the next succeeding address  !
;	      line, a 32-bit pattern of '00000000'h is written (NOT verif-!
;	      ied), scanning the base  address for data integrity FFFFFFFF!
;	      h and NO parity errors. The testing proceeds with locations !
;	      defined by each succeeding address line (until END of SRAM) !
;	      written with the '00000000'h pattern, each time scanning the!
;	      base  address for the 'FFFFFFFF'h pattern and verifying NO  !
;	      parity errors occurred. In this manner each succeeding addr-!
;	      ess line is verified to NOT conflict with the address under !
;	      test. The base   address is moved to the next succeeding	  !
;	      address line, and the 'FFFFFFFF'h data pattern is written to!
;	      this address. The algorithm proceeds repeatly until the add-!
;	      ress under test reaches END of SRAM.			  !
;									  !
; BEGIN 								  !
;     FOR current bank = SRAM						  !
;	 FOR base    address = 04h, 08h, 10h, 20h, 40h, 80h, 100h, 200h,  !
;			       80h, ......., 4000h, 8000h		  !
;	     current pattern = FFFFFFFFh				  !
;	     reverse pattern = 00000000h				  !
;	     write current pattern to current address			  !
;									  !
;	    FOR succeeding address = 08h, 10h, 20h, 40h, ....., 8000h	  !
;		write reverse pattern to succeeding address		  !
;		read/verify current address		   ERROR 0D	  !
;		check for memory no parity error	   ERROR 0E	  !
;	    NEXT succeeding address					  !
;									  !
;	 NEXT base address						  !
;     NEXT bank 							  !
;									  !
;     base address = 0000:0000h 					  !
;     write 'FFFFFFFF'h to base address 				  !
;     FOR current bank = SRAM						  !
;	 FOR current address = 04h, 08h, 10h, 20h, ......, 8000h	  !
;	    FOR current pattern = 00000000h, 01010101h			  !
;		write current pattern to current address		  !
;		read/verify base address		   ERROR 0F	  !
;		check for memory no parity error	   ERROR 10	  !
;	    NEXT pattern						  !
;	 NEXT address							  !
;     NEXT bank 							  !
; END									  !
;									  !
; ERRORS:								  !
; 0D Address Bus short							  !
;	  EAX= read data    EBX= reverse pattern   DS:SI= current address !
;	  FS= error code    ES:[BP]= succeeding address 		  !
;									  !
; 0F Address Bus Open							  !
;	  EAX= read data    EBX= expected data	   DS:SI= current address !
;	  FS= error code						  !
;									  !
; 0E & 10 Memory error, may be parity or circuit			  !
;	  AX= 4, SRAM parity err		   DS:SI= base address	  !
;	  FS= error code						  !
;-------------------------------------------------------------------------+

TST4:
	MOV	EBX,0		; set reverse pattern for write
	MOV	DS,BX		; sram memory size from 0 to 3:FFFF
	MOV	SI,04		; set first base address
	MOV	BP,08		; set succeeding address

TST4LOP1:
	MOV	EAX,0FFFFFFFFH	; get current pattern
	MOV	[SI],EAX	; write 'FFFFFFFF' to base address
	MOV	CX,0		; reset ES:BP= 0:BP
	MOV	ES,CX		;
TST4LOP2:
	MOV	AX,0DH		; setup error code for address check
	MOV	FS,AX		; error code in FS
	MOV	ES:[BP],EBX	; write reverse data to succeeding address
	MOV	EAX,[SI]	; read current data from current address
	CMP	EAX,0FFFFFFFFH	; verify data
	JNE	TST4_ERR	; jump if bad (no jump is fast)

	MOV	AX,0EH		; setup error code for parrity check
	MOV	FS,AX		; error code in FS
	MOV	DI,CPUST	; point to CPU status port
	MOV	AX,CS:[DI]	; read CPU status
	TEST	AX,01000000B	; keep D6, mask out other bits
	JNZ	TST4_ERR	; if not zero, we have parrity error

	SHL	BP,1		; get next succeeding address
	JNC	TST4LOP2	; jump if succeeding address < 8000h
TST4LOP3:
	MOV	CX,0800h	; convert DS:SI address to ES:BP
TST4LOP4:
	SHL	CX,1		; point to next bank
	CMP	CX,3000H	; last address is 2:0000h
	JNL	TST4LOP5	; go and write next base address

	MOV	ES,CX		; assume BP = 0000h
	MOV	AX,0DH		; setup error code for address check
	MOV	FS,AX		; error code in FS
	MOV	ES:[BP],EBX	; write reverse data to succeeding address
	MOV	EAX,[SI]	; read current data from current address
	CMP	EAX,0FFFFFFFFH	; verify data
	JNE	TST4_ERR	; jump if bad (no jump is fast)

	MOV	AX,0EH		; setup error code for parrity check
	MOV	FS,AX		; error code in FS
	MOV	DI,CPUST	; point to CPU status port
	MOV	AX,CS:[DI]	; read CPU status
	TEST	AX,01000000B	; keep D6, mask out other bits
	JNZ	TST4_ERR	; if not zero, we have parrity error
	JMP	TST4LOP4	; check succeding address in next bank

TST4LOP5:
	SHL	SI,1		; get next base address under test
	JC	TST4LOP6	; go write base address in bank addr
	MOV	BP,SI		;
	SHL	BP,1		; succeeding address setup
	JNC	TST4LOP1	; if SI< 8000h then loopback

	MOV	AX,0DH		; setup error code for address check
	MOV	FS,AX		; when error, FS is the error code
	MOV	EAX,0FFFFFFFFH	; get current pattern
	MOV	[SI],EAX	; write 'FFFFFFFF' as current data
	JMP	TST4LOP3	; go to write succeeding address test

TST4LOP6:
	MOV	AX,1000H	; base address is 1000:0h
	MOV	DS,AX
	MOV	AX,2000H	; cmp 1000:0h with 2000:0h
	MOV	ES,AX		; ES:BP = 2:0000h
	MOV	EAX,0FFFFFFFFH	; get current pattern
	MOV	[SI],EAX	; write 'FFFFFFFF' as current data

	MOV	AX,0DH		; setup error code for address check
	MOV	FS,AX		; when error, FS is the error code
	MOV	ES:[BP],EBX	; write reverse data to succeeding address
	MOV	EAX,[SI]	; read current data from base address
	CMP	EAX,0FFFFFFFFH	; verify data
	JNE	TST4_ERR	; jump if bad (no jump is fast)

	MOV	AX,0EH		; setup error code for parrity check
	MOV	FS,AX		; error code in FS
	MOV	DI,CPUST	; point to CPU status port
	MOV	AX,CS:[DI]	; read CPU status
	TEST	AX,01000000B	; keep D6, mask out other bits
	JNZ	TST4_ERR	; if not zero, we have parrity error
				; end of address line short test

	MOV	DS,BX		; reset DS:[SI] = 0:0
	MOV	ES,BX		; reset ES:[BP] = 0:0004
	MOV	SI,0		; set memory address pointer
	MOV	BP,04		; set succeeding address
	MOV	EAX,0FFFFFFFFH	; get current pattern
	MOV	EDX,01010101H	; data with inverted parity on byte basis.
	MOV	[SI],EAX	; write 'FFFFFFFF' as current data

TST4LOP7:
	MOV	AX,0FH		; setup error code for address check
	MOV	FS,AX		; error code in FS
	MOV	ES:[BP],EBX	; write reverse data to succeeding address
	MOV	EAX,[SI]	; read current data from base address
	CMP	EAX,0FFFFFFFFH	; verify data
	JNE	TST4_ERR	; jump if bad (no jump is fast)

	MOV	AX,10H		; setup error code for parrity check
	MOV	FS,AX		; error code in FS
	MOV	DI,CPUST	; point to CPU status port
	MOV	AX,CS:[DI]	; read CPU status
	TEST	AX,01000000B	; keep D6, mask out other bits
	JNZ	TST4_ERR	; if not zero, we have parrity error

	MOV	AX,0FH		; setup error code for address check
	MOV	FS,AX		; error code in FS
	MOV	ES:[BP],EDX	; write another data to succeeding address
	MOV	EAX,[SI]	; read current data from base address
	CMP	EAX,0FFFFFFFFH	; verify data
	JNE	TST4_ERR	; jump if bad (no jump is fast)

	MOV	AX,10H		; setup error code for parrity check
	MOV	FS,AX		; error code in FS
	MOV	DI,CPUST	; point to CPU status port
	MOV	AX,CS:[DI]	; read CPU status
	TEST	AX,01000000B	; keep D6, mask out other bits
	JNZ	TST4_ERR	; if not zero, we have parrity error

	SHL	BP,1		; get next succeeding address
	JNC	TST4LOP7	; jump if succeeding address < 8000h
	MOV	CX,0800H	; set succeeding address = bank address

TST4LOP8:
	SHL	CX,1		; point to next bank
	CMP	CX,3000H	; last address is 2000:0h
	JNL	TST5		; end of SRAM address open check
	MOV	ES,CX		; assume SI= 0000h
	MOV	AX,0FH		; setup error code for address check
	MOV	FS,AX		; error code in FS
	MOV	ES:[SI],EBX	; write reverse data to succeeding address
	MOV	EAX,[SI]	; read current data from base address
	CMP	EAX,0FFFFFFFFH	; verify data
	JNE	TST4_ERR	; jump if bad (no jump is fast)

	MOV	AX,10H		; setup error code for parrity check
	MOV	FS,AX		; error code in FS
	MOV	DI,CPUST	; point to CPU status port
	MOV	AX,CS:[DI]	; read CPU status
	TEST	AX,01000000B	; keep D6, mask out other bits
	JNZ	TST4_ERR	; if not zero, we have parrity error

	MOV	AX,0FH		; setup error code for address check
	MOV	FS,AX		; error code in FS
	MOV	ES:[SI],EDX	; write another data to succeeding address
	MOV	EAX,[SI]	; read current data from base address
	CMP	EAX,0FFFFFFFFH	; verify data
	JNE	TST4_ERR	; jump if bad (no jump is fast)

	MOV	AX,10H		; setup error code for parrity check
	MOV	FS,AX		; error code in FS
	MOV	DI,CPUST	; point to CPU status port
	MOV	AX,CS:[DI]	; read CPU status
	TEST	AX,01000000B	; keep D6, mask out other bits
	JNZ	TST4_ERR	; if not zero, we have parrity error
	JMP	TST4LOP8	; check succeding address in next bank


TST4_ERR:
	MOV	DI,CPUFLG	; select I/O port, direct memory mapping
	MOV	CX,08H		; D3= 1, cpu led on; D3= 0, cpu led off
	MOV	CS:[DI],CX	; turn CPU LED on as an error ocurred

	MOV	DI,CPUST	; point to CPU status I/O port
	MOV	BP,CS:[DI]	; read	CPU sataus
	TEST	BP,200h 	; D9=0 loop on BIT is requested
	JZ	TST4		; retry this test again

	JMP	$		; hang here if error

;-------------------------------------------------------------------------+
;[Test 5]  SRAM Data Fast Exchange Function TEST			  !
;									  !
;Purpose:  To test ability of all SRAM memory cells to hold a zero or a   !
;	   one and detec dynamic data/address faults.			  !
;									  !
;Description: The test is very fast because it takes advantage of various !
;	   CPU instructions. Three passes from the first to last memory   !
;	   address of SRAM are required to verify address and data faults !
;	   on both the data and parity SRAM.				  !
;									  !
; BEGIN 								  !
;     FOR SRAM first to last by 4's Do					  !
;	  write address = "FFFFFFFF"					  !
;     END (for SRAM)							  !
;									  !
;     FOR SRAM first to last by 4's Do					  !
;	  exchange "FFFFFFFF" with "00000000"				  !
;	  verify read data = "FFFFFFFF" 		   ERROR 11	  !
;	  read pattern "00000000"					  !
;	  verify read data = "00000000" 		   ERROR 11	  !
;									  !
;	  exchange "00000000" with "F4F4F4F4"				  !
;	  verify read data = "00000000" 		   ERROR 11	  !
;	  read pattern "F4F4F4F4F"					  !
;	  verify read data = "F4F4F4F4" 		   ERROR 11	  !
;     END (for SRAM)							  !
;									  !
;     Verify parity error				   ERROR 12	  !
; END									  !
;									  !
; ERRORS:								  !
; 11  Address Bus or Memory Data error					  !
;     EAX= read data	EBX= new write data	DS:SI= current or next	  !
;      FS= error code	EBP= last written data	       address		  !
;									  !
; 12  Memory error, may be parity or circuit				  !
;     AX= 4, SRAM parity err						  !
;     FS= error code							  !
;-------------------------------------------------------------------------+

TST5_ERR:
	MOV	DI,CPUFLG	; select I/O port, direct memory mapping
	MOV	CX,08H		; D3= 1, cpu led on; D3= 0, cpu led off
	MOV	CS:[DI],CX	; turn CPU LED on as an error ocurred

	MOV	DI,CPUST	; point to CPU status I/O port
	MOV	BP,CS:[DI]	; read	CPU sataus
	TEST	BP,200h 	; D9=0 loop on BIT is requested
	JZ	TST5		; retry this test again

	JMP	$		; hang here if error

TST5:
	MOV	BX,0		; memory range from 0:0000 to 3000:FFFF

ADB1_LOOP:
	MOV	ES,BX
	MOV	DI,0		; set 1st address
	MOV	CX,4000H	; memory size in dwords
	MOV	EAX,0FFFFFFFFH	; set flood data pattern
	CLD			; clear dir flag so string instr incrmnt DI
	REP	STOSD		; flood segment

	ADD	BX,1000H	; next bank
	CMP	BX,4000H	; last SRAM is 3:FFFFh
	JL	ADB1_LOOP	;

	MOV	AX,11H		; setup error code
	MOV	FS,AX		; when error, FS is the error code
	MOV	EBP,0FFFFFFFFH	; set last written pattern
	MOV	EBX,0		; set revers pattern for write
	MOV	EDX,0		; sram memory size from 0 to 3:FFFF

ADB2_LOOP:
	MOV	DS,DX
	MOV	SI,0		; set memory address pointer
	MOV	CX,4000H	; (1024X64)/4 loop count for double word
	MOV	EAX,EBX 	; get revers pattern

ADB3_LOOP:			; loop here 16K times
	XCHG	[SI],EAX	; read 'FFFFFFFF',write '00000000'
	CMP	EAX,EBP 	; verify data
	JNE	TST5_ERR	; jump if bad (no jump is fast)

	LODSD			; read '0000' back and inc SI by 4
	CMP	EAX,EBX 	; verify data

	LOOPZ	ADB3_LOOP	; loop as long as CX not 0 and no error.
	JNE	TST5_ERR	; jump if bad (no jump is fast)

	MOV	AX,12H		; setup error code for parrity check
	MOV	FS,AX		; error code in FS = 4
	MOV	DI,CPUST	; point to CPU status port
	MOV	AX,CS:[DI]	; read CPU status
	TEST	AX,01000000B	; keep D6, mask out other bits
	JNZ	TST5_ERR	; if not zero, we have parrity error

	MOV	AX,11H		; setup error code for memory test
	MOV	FS,AX		; error code in FS = 3
	ADD	DX,1000H	; next bank
	CMP	DX,4000H	; last SRAM is 3:FFFF
	JL	ADB2_LOOP	; jmp if DX < 4000

	MOV	EBP,0		; set last written pattern
	MOV	EBX,0F4F4F4F4H	; after test, all address should be 'F4'
	MOV	EDX,0		; SRAM mmry size from 0 to 4000:0

ADB4_LOOP:
	MOV	DS,DX
	MOV	SI,0		; set memory address pointer
	MOV	CX,4000H	; (1024X64)/4 loop count for double word
	MOV	EAX,EBX 	; get revers pattern

ADB5_LOOP:			; loop here 16K times
	XCHG	[SI],EAX	; read '00000000', write 'F4F4F4F4'
	CMP	EAX,EBP 	; verify data
	JNE	TST5_ERR	; jump if bad (no jump is fast)

	LODSD			; read 'F4F4F4F4' back and inc SI by 4
	CMP	EAX,EBX 	; verify data

	LOOPZ	ADB5_LOOP	; loop as long as CX not 0 and no error.
	JNE	TST5_ERR	; jump if bad (no jump is fast)

	MOV	AX,12H		; setup error code for parrity check
	MOV	FS,AX		; error code in FS = 4
	MOV	DI,CPUST	; point to CPU status port
	MOV	AX,CS:[DI]	; read CPU status
	TEST	AX,01000000B	; keep D6, mask out other bits
	JNZ	TST5_ERR	; if not zero, we have parrity error

	MOV	AX,11H		; setup error code for memory test
	MOV	FS,AX		; error code in FS = 3
	ADD	DX,1000H	; next bank
	CMP	DX,4000H	; last SRAM is 3:FFFF
	JL	ADB4_LOOP	; jmp if DX < 4000



;----- PROTECTED MODE INIT ---------------------------------------------+
;MOV SCRATCH GDT TO RAM AT PHYSICAL LOCATION 0				!
;-----------------------------------------------------------------------+

PRO_TE:
	cld
	xor	di,di
	mov	es,di
	mov	si,offset temp_gdt
	mov	cx,end_temp_gdt - temp_gdt
	inc	cx
	rep	movs byte ptr es:[di],byte ptr cs:[si]
;load gdtr for segment	-- temp_gdt
	lgdt	tgdt_pword

;SWITCH TO PROTECTED MODE
        MOV     EAX,CR0
	MOV	EAX,1		; set PE bit at CR0 register
	MOV	CR0,EAX
	JMP	SHORT FLUSH	; use short jump to flush regesters

FLUSH:				; now entering protect mode
	MOV	BX,08H
	MOV	DS,BX		; ds,es,ss,all point to temp.gdt[1]
	MOV	ES,BX
	MOV	SS,BX
	MOV	ESP,40000H	; setup stack at bottom of SRAM

	CALL	ENABLECRT	; now display some messages on MXE ch0
	MOV	AL,03H		; ascii 03 is clean screen
	CALL	DISPLAYONE
	MOV	SI,OFFSET COPYRIGHT
	OR	ESI,0FFFF0000H	; ESI point to string first address
	MOV	ECX,AFTER_COPYRIGHT - COPYRIGHT
	CALL	DISPLAYMORE	; display copyright statement

	MOV	DI,CPUST	; point to CPU status I/O port
	MOV	BP,CS:[DI]	; read	CPU sataus
	TEST	BP,200h 	; D9=1 loop on BIT is requested
	JNZ	MSGUPDATE	; in run-in loop, update message only

	MOV	SI,OFFSET SRAMLOOP
	OR	ESI,0FFFF0000H	; ESI point to string first address
	MOV	ECX,ENDSRAMLOOP - SRAMLOOP	;count bytes
	CALL	DISPLAYMORE	; display sram size at 3rd line

	MOV	AX,2000H	; RTC RAM 20h contains loopcount low byte
	CALL	WRITERTC
	MOV	AX,2100H	; RTC RAM 21h contains loopcount hi byte
	CALL	WRITERTC	; reset loopcount

	JMP	TST6

MSGUPDATE:
	MOV	SI,OFFSET SRAMSIZE
	OR	ESI,0FFFF0000H	; ESI point to string first address
	MOV	ECX,ENDSRAMSIZE - SRAMSIZE	;count bytes
	CALL	DISPLAYMORE	; display sram size at 3rd line

;-------------------------------------------------------------------------+
;[Test 6]  DRAM Data Bus Test						  !
;									  !
; Purpose: To verify that no shorts or opens exist on DRAM data bus.	  !
;	   Some faults with the DRAM parity logic will be detected.	  !
;	   It also stores memory size at 1st byte of RTC user RAM.	  !
;									  !
; Description: Walking one and zero data patterns (32-bit) are written,   !
;	   then read/verified for correct data and NO parity error, on	  !
;	   bank basis. All patterns has an odd number of 'high'(1) bits.  !
;	   A double-bit rotating pattern, starting value '00000003'H, will!
;	   be written/read/verified for correct data and NO parity. These !
;	   patterns have an even number of 'high'(1) bits.		  !
;	   The routine uses address zero of DRAM of each 64KB bank,	  !
;	   so chip faults at that location will be flagged as well.	  !
;	   At each end of MB size, store the memory size to RTC.	  !
;									  !
; BEGIN 								  !
;     FOR current bank = 40000h, 50000h, ....., 730000h 		  !
;	  current address = first in current bank			  !
;	 FOR current pattern = FFFFFFFF, 01, 02, 04, ...80000000, 0,	  !
;			       7FFFFFFF, BFFFFFFF, ......., FFFFFFFE	  !
;	     write current pattern to current address			  !
;	     read/verify current address		ERROR 20	  !
;	     check for memory parity error		ERROR 21	  !
;	 NEXT pattern							  !
;     NEXT bank 							  !
;									  !
;     FOR current bank = 40000h, 50000h, ..., 730000h			  !
;	  current address = first in current bank			  !
;	 FOR current pattern = 03, 06, 0C, 18, 30,...., C0000000, 80000001!
;	     write current pattern to current address			  !
;	     read/verify current address		ERROR 22	  !
;	     check for memory parity error		ERROR 23	  !
;	 NEXT pattern							  !
;     NEXT bank 							  !
; END									  !
;									  !
; ERRORS:								  !
; 20 & 22 Data bus or memory data error 				  !
;	  EAX= read data    EBX= expected data	   DS:SI= current address !
;	  17th byte of RTC user RAM = error code			  !
;									  !
; 21 & 23 Memory error, may be parity or circuit			  !
;	  AX= 4, SRAM parity err					  !
;	  17th byte of RTC user RAM = error code			  !
;-------------------------------------------------------------------------+
TST6:
	MOV	AL,0		; assume the DRAM is not existing
	MOV	EDI,RTCLOCK	; point to RTC address select port
	MOV	AH,0Eh		; point to 1st byte on user RAM
	MOV	[EDI],AH	;
	MOV	EDI,RTCLOC2	; point to RTC data r/w port
	MOV	[EDI],AL	; write DRAM size to RTC RAM

	MOV	ESI,040000H	; start from address 040000H
TST6LOP1:
	MOV	EDI,RTCLOCK	; point to RTC address select port
	MOV	AH,1Eh		; point to 17th byte on user RAM
	MOV	[EDI],AH	;
	MOV	EDI,RTCLOC2	; point to RTC data r/w port
	MOV	AL,20h		; setup error code
	MOV	[EDI],AL	; write error code to RTC RAM

	MOV	EBX,0FFFFFFFFH	; current data pattern
	MOV	EAX,01		; next pattern
	MOV	EDX,EAX 	; save next pattern
	MOV	ECX,33		; 33 patterns from FFFFFFFF to 01h
	MOV	[ESI],EBX	; write current pattern to current address

TST6LOP2:
	XCHG	[ESI],EAX	; read data from currnt addr & write next
	CMP	EAX,EBX 	; verify read data against current pattern
	JNE	TST6SIZE	 ; jump if bad (no jump is fast)

	MOV	EBX,EDX 	; save current pattern
	SHL	EDX,1		; setup next data pattern
	MOV	EAX,EDX 	; ready to write data pattern from EAX reg
	DEC	CX
	JNZ	TST6LOP2	; loop for next pattern test

	MOV	EAX,7FFFFFFFH	; next data pattern 7FFFFFFF
	MOV	EDX,EAX 	; prepare for shift right
	XCHG	[ESI],EAX	; read data from currnt addr & write next
	CMP	EAX,EBX 	; verify read data against current pattern
	JNE	TST6SIZE	 ; jump if bad (no jump is fast)

	MOV	EBX,EDX 	; save current pattern for compare later
	ROR	EDX,1		; setup next data pattern
	MOV	EAX,EDX
	MOV	CX,31		; 31 patterns from 7FFFFFFF to FFFFFFFE

TST6LOP3:
	XCHG	[ESI],EAX	; read data from currnt addr & write next
	CMP	EAX,EBX 	; verify read data against current pattern
	JNE	TST6SIZE	 ; jump if bad (no jump is fast)

	MOV	EBX,EDX 	; save current pattern for compare later
	ROR	EDX,1		; setup next data pattern
	MOV	EAX,EDX
	DEC	CX
	JNZ	TST6LOP3	; loop for next pattern test

	MOV	EDI,RTCLOCK	; point to RTC address select port
	MOV	AH,1Eh		; point to 17th byte on user RAM
	MOV	[EDI],AH	;
	MOV	EDI,RTCLOC2	; point to RTC data r/w port
	MOV	AL,21h		; setup error code
	MOV	[EDI],AL	; write error code to RTC RAM

	MOV	EBX,CPUST	; point to CPU status port
	MOV	AX,CS:[BX]	; read CPU status
	AND	AX,11000000B	; keep D6 & D7, mask out other bits
	CMP	AX,0		; D6 & D& should be all zero
	JNE	TST6ERR 	 ; yes, all zero; no parrity error

	ADD	ESI,1000H	; point to next 64KB bank
	CMP	ESI,840000H	; 83FFFFH is last byte of DRAM
	JL	TST6LOP1	;

TST6SIZE:
	TEST	ESI,0FFFFH	; Is this address within 64K bank?
	JNZ	TST6ERR 	; Yes, it is not 1st byte on 64KB
	TEST	ESI,0B0000H	; Is this 1st address of each MB?
	JNZ	TST6ERR 	; No, it is not 1st byte on MB
	TEST	ESI,040000H	; Is this 1st address of each MB?
	JZ	TST6ERR 	; No, it is not 1st byte on MB
	MOV	EAX,ESI 	; Good, this address is 1st byte
	SHR	EAX,20		; scratch out low 20 bits
	TEST	EAX,0FFh	; make sure the DRAM existing
	JZ	TST6ERR 	; bad, it is 1st byte of 1MB
	TEST	EAX,1		; DRAM odd size?
	JNZ	TST6SIZEX	; yes, it is odd MB size
TST6SIZEOK:
	MOV	EDI,RTCLOCK	; point to RTC address select port
	MOV	AH,0Eh		; point to 1st byte on user RAM
	MOV	[EDI],AH	;
	MOV	EDI,RTCLOC2	; point to RTC data r/w port
	MOV	[EDI],AL	; write DRAM size to RTC RAM

	MOV	AL,44H		; show 'D' of 'DRAM' as test6 end
	CALL	DISPLAYONE
	JMP	TST7		; go on next test

TST6SIZEX:
	CMP	EAX,1		; DRAM = 1MB size?
	JE	TST6SIZEOK	; 1MB size is legal

TST6ERR:
	MOV	EDI,CPUFLG	; select I/O port, direct memory mapping
	MOV	CX,08H		; D3= 1, cpu led on; D3= 0, cpu led off
	MOV	CS:[DI],CX	; turn CPU LED on

	MOV	DI,CPUST	; point to CPU status I/O port
	MOV	BP,CS:[DI]	; read	CPU sataus
	TEST	BP,200h 	; D9=0 loop on BIT is requested
	JZ	TST6		; retry this test again

	JMP	$		; hang here if error

;-------------------------------------------------------------------------+
;[Test 7] DRAM Parity Generator/Check Test				  !
;									  !
; Purpose: To verify DRAM parity error circuity.			  !
;									  !
; Description: The test first writes a 32-bit pattern with an odd number  !
;	   of bits 'high'(1) in each byte and normal parity generation to !
;	   the lowest address in the bank. The data is then read/verified !
;	   and parity status checked for NO error. The FORCED parity fun- !
;	   ction is enabled which causes the EVEN parity logic to generate!
;	   a 'high'(1) on the four outputs, then read/verified again, with!
;	   a check to ensure parity error occurred. The FORCED parity fun-!
;	   ction is then disabled and the parity error is reset. The pat- !
;	   tern is then rotated and the algorithm is repeated within each !
;	   byte to test all the lines to the parity generator/checker	  !
;	   chips. Although 32-bit writes are performed in this test, each !
;	   byte is read back for test separately, starting from the least !
;	   significant byte and processing to the most significant byte   !
;	   of the 4-byte double-word.					  !
;									  !
; BEGIN 								  !
;     FOR current bank = DRAM						  !
;	  current address = first in current bank			  !
;	 FOR current pattern = 01010101h, 02020202h, 04040404h, .....	  !
;			       20202020h, 40404040h, 80808080h, 	  !
;	     write current pattern to current address			  !
;	     read/verify current address		ERROR 24	  !
;	     check for memory no parity error		ERROR 25	  !
;	     enable forced parity function				  !
;	     read/verify current address		ERROR 26	  !
;	     check for memory parity error occurred	ERROR 27	  !
;	     disable forced parity function				  !
;	 NEXT pattern							  !
;     NEXT bank 							  !
; END									  !
;									  !
; ERRORS:								  !
; 24 & 26 Data bus or memory data error 				  !
;	  EAX= read data    EBX= expected data	   DS:SI= current address !
;	  17th byte of RTC user RAM = error code			  !
;									  !
; 25 & 27 Memory error, may be parity or circuit			  !
;	  AX= 80,DRAM parity err					  !
;	  17th byte of RTC user RAM = error code			  !
;-------------------------------------------------------------------------+

TST7_ERR:
	MOV	EDI,CPUFLG	; select I/O port, direct memory mapping
	MOV	CX,08H		; D3= 1, cpu led on; D3= 0, cpu led off
	MOV	CS:[DI],CX	; turn CPU LED on

	MOV	DI,CPUST	; point to CPU status I/O port
	MOV	BP,CS:[DI]	; read	CPU sataus
	TEST	BP,200h 	; D9=0 loop on BIT is requested
	JZ	TST7		; retry this test again

	JMP	$		; hang here if error

TST7:
;********** for old PCA without disable NMI ****************************+
;;;;	JMP	FOROLDPCA	; by pass parity check, temperary	*
;***********************************************************************+

	MOV	ESI,40000H	; set 1st address of DRAM
	MOV	BX,CPUST	; CS:[BX]= CPU status port
	MOV	DI,CPUFLG	; CS:[DI]= CPU flag I/O port
	MOV	EDX,01010101H	; current data pattern, odd '1'

TST7LOP1:
	MOV	[ESI],EDX	; write current pattern to current address
	MOV	AX,1E24H	; write 24h err code to 1Eh of RTC RAM
	CALL	WRITERTC	;
	MOV	AL,[ESI]	; read data from currnt addr first byte
	CMP	AL,DL		; verify read data against current pattern
	JNE	TST7_ERR	; jump if bad (no jump is fast)

	MOV	AX,1E25H	; writ err code 25h to RTC user RAM
	CALL	WRITERTC
	MOV	AX,CS:[BX]	; read CPU status
	TEST	AX,10000000B	; keep D7, mask out other bits
	JNZ	TST7_ERR	; if D7 not zero; parrity error ocurred

	MOV	AX,1E24H	; write err code 24 to RTC RAM
	CALL	WRITERTC
	INC	ESI
	MOV	AL,[ESI]	; read data from second byte
	CMP	AL,DL		; verify read data against current pattern
	JNE	TST7_ERR	; jump if bad (no jump is fast)

	MOV	AX,1E25H	; setup error code
	CALL	WRITERTC	;
	MOV	AX,CS:[BX]	; read CPU status
	TEST	AX,10000000B	; keep D7, mask out other bits
	JNZ	TST7_ERR	; if D7 not zero; parrity error ocurred

	MOV	AX,1E24H	; point to 17th byte on user RAM
	CALL	WRITERTC	;
	INC	ESI
	MOV	AL,[ESI]	; read data from third byte
	CMP	AL,DL		; verify read data against current pattern
	JNE	TST7_ERR	; jump if bad (no jump is fast)

	MOV	AX,1E25H	; point to 17th byte on user RAM
	CALL	WRITERTC	;
	MOV	AX,CS:[BX]	; read CPU status
	TEST	AX,10000000B	; keep D7, mask out other bits
	JNZ	TST7_ERR	; if D7 not zero; parrity error ocurred

	MOV	AX,1E24H	; point to 17th byte on user RAM
	CALL	WRITERTC	;
	INC	ESI
	MOV	AL,[ESI]	; read data from fourth byte
	CMP	AL,DL		; verify read data against current pattern
	JNE	TST7_ERR	; jump if bad (no jump is fast)

	MOV	AX,1E25H	; point to 17th byte on user RAM
	CALL	WRITERTC	;
	MOV	AX,CS:[BX]	; read CPU status
	TEST	AX,10000000B	; keep D7, mask out other bits
	JNZ	TST7_ERR	; if D7 not zero; parrity error ocurred

	MOV	CX,01H		; D0= 1, force parity generate error
	MOV	CS:[DI],CX	; we expect parity error at next read

	MOV	AX,1E26H	; write 26h err code to RTC user RAM
	CALL	WRITERTC	;
	MOV	ESI,40000H	; point to first address
	MOV	[ESI],EDX	; write in order to get wrong parity
	MOV	AL,[ESI]	; read data from currnt addr first byte
	CMP	AL,DL		; verify read data against current pattern
	JNE	TST7_ERR	; jump if bad (no jump is fast)

	MOV	AX,1E27H	; setup err code 27h at RTC user RAM
	CALL	WRITERTC	;
	MOV	AX,CS:[BX]	; read CPU status
	TEST	AX,10000000B	; keep D7, mask out other bits
	JZ	TST7_ERR	; if D7 not zero; parrity error ocurred
	MOV	CX,02H		; D1= 1, clear parity error
	MOV	CS:[DI],CX	; expect parity error at next read
	MOV	AX,CS:[BX]	; read CPU status
	TEST	AX,10000000B	; keep D7, mask out other bits
	JNZ	TST7_ERR	; D7 not zero; parrity error can't reset
	MOV	CX,0		; D1= 0, not clear parity error
	MOV	CS:[DI],CX	; expect no parity error at next read

	MOV	AX,1E26H	; point to 17th byte on user RAM
	CALL	WRITERTC	;
	INC	ESI
	MOV	AL,[ESI]	; read data from second byte
	CMP	AL,DL		; verify read data against current pattern
	JNE	TST7_ERR	; jump if bad (no jump is fast)

	MOV	AX,1E27H	; point to 17th byte on user RAM
	CALL	WRITERTC	;
	MOV	AX,CS:[BX]	; read CPU status
	TEST	AX,10000000B	; keep D7, mask out other bits
	JZ	TST7_ERR	; if D7 not zero; parrity error ocurred
	MOV	CX,02H		; D1= 1, clear parity error; D0=0, normal
	MOV	CS:[DI],CX	; reset parity error at CPUSTU D7 bit
	MOV	AX,CS:[BX]	; read CPU status
	TEST	AX,10000000B	; keep D7, mask out other bits
	JNZ	TST7_ERR	; D7 not zero; parrity error can't reset
	MOV	CX,0		; D1= 0, not clear parity error
	MOV	CS:[DI],CX	; expect parity error at next read

	MOV	AX,1E26H	; point to 17th byte on user RAM
	CALL	WRITERTC	;
	INC	ESI
	MOV	AL,[ESI]	; read data from third byte
	CMP	AL,DL		; verify read data against current pattern
	JNE	TST7_ERR	; jump if bad (no jump is fast)

	MOV	AX,1E27H	; point to 17th byte on user RAM
	CALL	WRITERTC	;
	MOV	AX,CS:[BX]	; read CPU status
	TEST	AX,10000000B	; keep D7, mask out other bits
	JZ	TST7_ERR	; if D7 not zero; parrity error ocurred
	MOV	CX,02H		; D1= 1, clear parity error; D0=0, normal
	MOV	CS:[DI],CX	; reset parity error at CPUSTU D7 bit
	MOV	AX,CS:[BX]	; read CPU status
	TEST	AX,10000000B	; keep D7, mask out other bits
	JNZ	TST7_ERR	; D7 not zero; parrity error can't reset
	MOV	CX,0		; D1= 0, not clear parity error
	MOV	CS:[DI],CX	; expect no parity error at next read

	MOV	AX,1E26H	; point to 17th byte on user RAM
	CALL	WRITERTC	;
	INC	ESI
	MOV	AL,[ESI]	; read data from fourth byte
	CMP	AL,DL		; verify read data against current pattern
	JNE	TST7_ERR	; jump if bad (no jump is fast)
	MOV	AX,CS:[BX]	; read CPU status
	TEST	AX,10000000B	; keep D7, mask out other bits
	JZ	TST7_ERR	; if D7 not zero; parrity error ocurred
	MOV	CX,02H		; D1=1,clear parity err;D0=0,back to normal
	MOV	CS:[DI],CX	; reset parity error at CPUSTU D7 bit
	MOV	AX,CS:[BX]	; read CPU status
	TEST	AX,10000000B	; keep D7, mask out other bits
	JNZ	TST7_ERR	; D7 not zero; parrity error can't reset
	MOV	CX,0		; D1= 0, not clear parity error
				; D0= 0, normal parity generate
	MOV	CS:[DI],CX	; expect no parity error at next read


	MOV	SI,0		; reset low address back to 1st of bank
	SHL	EDX,1		; setup next data pattern
	JNC	TST7LOP1	; if carry, all odd '1' pattern passed
FOROLDPCA:
	MOV	AL,52H		; show 'R' of 'DRAM' as test7 end
	CALL	DISPLAYONE
;	JMP	TST8		; end of parity circult testing

;-------------------------------------------------------------------------+
;[Test 8] DRAM Byte Decode Address Line Test				  !
;									  !
; Purpose: To check for shorts and opens on the DRAM byte decode address  !
;	   lines.							  !
;									  !
; Description: Data is writen and verified to the beginning of each bank  !
;	   in byte and word patterns. Doubleword patterns have been tested!
;	   in the [Test 6] DRAM Data Line Test. The 80386 stores word data!
;	   in memory in reverse byte order i.e. the high-order byte follo-!
;	   wing the low-order byte. Therefore the high (most-significant) !
;	   byte of a data word is in the highest memory address. The word !
;	   read/write can be performed at both even and odd address.	  !
;									  !
; BEGIN 								  !
;     FOR current bank = DRAM						  !
;	  write 00h, 01h, 02h, 03h for the offset addresses 00h, 01h, 02h,!
;		03h respectively.					  !
;	  read/verify data 00h, 01h, 02h, 03h for the offset addresses	  !
;		00h, 01h, 02h, 03h respectively.	     ERROR 28	  !
;	  write 03h, 02h, 01h, 00h for the offset addresses 00h, 01h, 02h,!
;		03h respectively.					  !
;	  read/verify data 03h, 02h, 01h, 00h for the offset addresses	  !
;		00h, 01h, 02h, 03h respectively.	     ERROR 29	  !
;	  write AA55h, CC33h for the offset address 00h, 02h respectively.!
;	  read/verify AA55h, CC33h for offset addresses 00h, 02h	  !
;		respectively.				     ERROR 2A	  !
;	  write 9966h for offset address 01h.				  !
;	  read/verify 6655h, 9966h, CC99h for offset addresses 00h, 01h,  !
;		02h respectively.			     ERROR 2B	  !
;     NEXT bank 							  !
; END									  !
;									  !
; ERRORS:								  !
; 28 & 29,  byte decode address error					  !
;	  AL= read data,   BX & CX=  expected data,  DS:SI= current addres!
;	  17th byte of RTC user RAM = error code			  !
; 2A  word decode address error at even boundry 			  !
;	  AX= read data,   BX & CX=  expected data,  DS:SI= current addres!
;	  17th byte of RTC user RAM = error code			  !
; 2B  word decode address error at odd boundry				  !
;	  AX= read data,   BX, CX & DX=  expected data,  DS:SI= current   !
;	  17th byte of RTC user RAM = error code		address   !
;-------------------------------------------------------------------------+
TST8:
	CALL	LASTADDR	; get last DRAM address back in EDX
	MOV	EBP,EDX 	; store it in EBP; EDX has other use
	MOV	ESI,40000H	; set 1st address of DRAM

TST8LOP1:
	MOV	BX,0100H	; byte patterns 00h, 01h, 02h, 03h
	MOV	CX,0302H	;
	MOV	AX,1E28H	; write 28h err code to RTC user RAM
	CALL	WRITERTC	;
TST8LOP2:
	MOV	[ESI],BL	 ; write byte 00h to address 0000h
	MOV	[ESI+1],BH	 ; write byte 01h to address 0001h
	MOV	[ESI+2],CL	 ; write byte 02h to address 0002h
	MOV	[ESI+3],CH	 ; write byte 03h to address 0003h
	MOV	AL,[ESI]	 ; read byte data from addrs 0000h
	CMP	AL,BL		; verify written byte data
	JNZ	TST8_ERR
	INC	ESI		 ; point to next byte address
	MOV	AL,[ESI]	 ; read byte data from addrs 0001h
	CMP	AL,BH		; verify written byte data
	JNZ	TST8_ERR
	INC	ESI		 ; point to next byte address
	MOV	AL,[ESI]	 ; read byte data from addrs 0002h
	CMP	AL,CL		; verify written byte data
	JNZ	TST8_ERR
	INC	ESI		 ; point to next byte address
	MOV	AL,[ESI]	 ; read byte data from addrs 0003h
	CMP	AL,CH		; verify written byte data
	JNZ	TST8_ERR

	MOV	SI,0		; reset offset
	CMP	AL,0		; are we first time here?
	JE	TST80		; yes, jump to word patterns test
	MOV	BX,0203H	; byte patterns 03h, 02h, 01h, 00h
	MOV	CX,0001H	;

	MOV	AX,1E29H	; setup err code 29h in RTC user RAM
	CALL	WRITERTC	;
	JMP	TST8LOP2

TST8_ERR:
	MOV	DI,CPUFLG	; select I/O port, direct memory mapping
	MOV	CX,08H		; D3= 1, cpu led on; D3= 0, cpu led off
	MOV	CS:[DI],CX	; turn CPU LED on as an error ocurred

	MOV	DI,CPUST	; point to CPU status I/O port
	MOV	BP,CS:[DI]	; read	CPU sataus
	TEST	BP,200h 	; D9=1 loop on BIT is requested
	JZ	TST8

	JMP	$		; hang here if error

TST80:
	ADD	ESI,10000H	; point to next 64KB bank
	CMP	ESI,EDX 	; last DRAM address is in EDX
	JL	TST8LOP1	; otherwise go on next test

	MOV	ESI,40000H	; word pattern test from 4000:0
TST8LOP3:
	MOV	AX,1E2AH	; setup srror code
	CALL	WRITERTC

	MOV	BX,0AA55H	; word patterns 55, AA, 33, CC,
	MOV	CX,0CC33H	;
	MOV	SI,0		; start from address ?:0000
	MOV	[ESI],BX	; write word 55, AA to addrs 00h
	MOV	[ESI+2],CX	; write word 33, CC to addrs 02h
	MOV	AX,[ESI]	; read word data from addrs 0000h
	CMP	AX,BX		; verify written word data
	JNZ	TST8_ERR
	ADD	ESI,2		; point to next word address
	MOV	AX,[ESI]	; read word data from addrs 0002h
	CMP	AX,CX		; verify written word data
	JNZ	TST8_ERR

	MOV	AX,1E2BH	; setup srror code
	CALL	WRITERTC
	MOV	EBX,6655H	; new data in byte series 55, 66,
	MOV	ECX,9966H	; 99, CC; in word serie are 6655,
	MOV	EDX,0CC99H	; 9966 and CC99.

	MOV	SI,0		; start from address 0:0000
	MOV	[ESI+1],CX	; write word to address 0001h
	MOV	AX,[ESI]	; read word data from addrs 0000h
	CMP	AX,BX		; verify written word data
	JNZ	TST8_ERR
	INC	ESI		; point to odd address for word
	MOV	AX,[ESI]	; read word data from addrs 0001h
	CMP	AX,CX		; verify written word data
	JNZ	TST8_ERR
	INC	ESI		; point to even address for word
	MOV	AX,[ESI]	; read word data from addrs 0002h
	CMP	AX,DX		; verify written word data
	JNZ	TST8_ERR

	ADD	ESI,10000H	; point to next 64KB bank
	CMP	ESI,EBP 	; last DRAM address is in EBP
	JL	TST8LOP3	; otherwise go on next test

	MOV	AL,41H		; show 'A' of 'DRAM' as test8 end
	CALL	DISPLAYONE
;-------------------------------------------------------------------------+
;[Test 9] DRAM Address Line Test					  !
;									  !
; Purpose: To verify DRAM address lines and check for shorts and opens on !
;	   the DRAM address bus and internal RAM chip addressing.	  !
;									  !
; Description:								  !
;	   Shorted Address Line Test for each bank:			  !
;	      A 32-bit data pattern, 'FFFFFFFF'h, is written to the 04h   !
;	      base address of SRAM bank. For the next succeeding address  !
;	      line, a 32-bit pattern of '00000000'h is written (NOT verif-!
;	      ied), scanning the base  address for data integrity FFFFFFFF!
;	      h and NO parity errors. The testing proceeds with locations !
;	      defined by each succeeding address line (until END of DRAM) !
;	      written with the '00000000'h pattern, each time scanning the!
;	      base  address for the 'FFFFFFFF'h pattern and verifying NO  !
;	      parity errors occurred. In this manner each succeeding addr-!
;	      ess line is verified to NOT conflict with the address under !
;	      test. The base   address is moved to the next succeeding	  !
;	      address line, and the 'FFFFFFFF'h data pattern is written to!
;	      this address. The algorithm proceeds repeatly until the add-!
;	      ress under test reaches END of DRAM.			  !
;									  !
; BEGIN 								  !
;     FOR current bank = DRAM						  !
;	 FOR base    address = 04h, 08h, 10h, 20h, 40h, 80h, 100h, 200h,  !
;			       80h, ......., 4000h, 8000h		  !
;	     current pattern = FFFFFFFFh				  !
;	     reverse pattern = 00000000h				  !
;	     write current pattern to current address			  !
;									  !
;	    FOR succeeding address = 08h, 10h, 20h, 40h, ....., 8000h	  !
;		write reverse pattern to succeeding address		  !
;		read/verify current address		   ERROR 2C	  !
;		check for memory no parity error	   ERROR 2D	  !
;	    NEXT succeeding address					  !
;									  !
;	 NEXT base address						  !
;     NEXT bank 							  !
;									  !
;     base address = 0000:0000h 					  !
;     write 'FFFFFFFF'h to base address 				  !
;     FOR current bank = DRAM						  !
;	 FOR current address = 04h, 08h, 10h, 20h, ......, 8000h	  !
;	    FOR current pattern = 00000000h, 01010101h			  !
;		write current pattern to current address		  !
;		read/verify base address		   ERROR 2E	  !
;		check for memory no parity error	   ERROR 2F	  !
;	    NEXT pattern						  !
;	 NEXT address							  !
;     NEXT bank 							  !
; END									  !
;									  !
; ERRORS:								  !
; 2C Address Bus short							  !
;	  EAX= read data    EBX= reverse pattern   DS:SI= current address !
;			    ES:[BP]= succeeding address 		  !
;	  17th byte of RTC user RAM = error code			  !
;									  !
; 2E Address Bus Open							  !
;	  EAX= read data    EBX= expected data	   DS:SI= current address !
;	  17th byte of RTC user RAM = error code			  !
;									  !
; 2D & 2F Memory error, may be parity or circuit			  !
;	  AX= 4, SRAM parity err		   DS:SI= base address	  !
;	  17th byte of RTC user RAM = error code			  !
;-------------------------------------------------------------------------+
TST9:
	CALL	LASTADDR	; get last address in EDX
	MOV	EBX,0		; set reverse pattern for write
	MOV	ESI,40004H	; set first base address in DRAM
	MOV	EBP,40008H	; set succeeding address in DRAM
	MOV	ECX,00008H	; offset of succeeding address

TST9LOP1:
	PUSH	ECX		; save it as next base address offset
	MOV	EAX,0FFFFFFFFH	; get current pattern
	MOV	[ESI],EAX	; write 'FFFFFFFF' to base address
TST9LOP2:
	MOV	AX,1E2CH	; setup error code for address check
	CALL	WRITERTC	; error code in RTC user RAM 17th byte
	MOV	[EBP],EBX	; write reverse data to succeeding address
	MOV	EAX,[ESI]	; read current data from current address
	CMP	EAX,0FFFFFFFFH	; verify data
	JNE	TST9_ERR	; jump if bad (no jump is fast)

	MOV	AX,1E2DH	; setup error code for parrity check
	CALL	WRITERTC	; error code in RTC user RAM 17th byte
	MOV	DI,CPUST	; point to CPU status port
	MOV	AX,CS:[DI]	; read CPU status
	TEST	AX,10000000B	; keep D7, mask out other bits
	JNZ	TST9_ERR	; if not zero, we have parrity error

	SHL	ECX,1		; get next succeeding address offset
	CMP	ECX,40000H	; do we need other algorithem?
	JNL	TST9LOP3	; yes, ECX = 40000H now
	MOV	EBP,ECX
	OR	EBP,40000H	; setup next succeeding address
	JMP	TST9LOP2	; jump if succeeding address < 8000h

TST9LOP3:
	MOV	EBP,ECX 	; get next succeeding address
	CMP	EBP,EDX 	; do we reach the last address?
	JL	TST9LOP2	; go and write next succeeding address

	POP	ECX		; get next base address offset back
	MOV	ESI,ECX
	CMP	ESI,40000H	; do we need other algorithem for ESI?
	JNL	TST9LOP5	; yes, we do.
	OR	ESI,40000H	; now we got next base address
	SHL	ECX,1		; prepare next succeeding address
	MOV	EBP,ECX 	;
	OR	EBP,40000H	;
	JMP	TST9LOP1	; go and write next base address

TST9LOP5:
	SHL	ECX,1		; we will use addr 40000h as open test
	MOV	ESI,ECX 	; This address is new base address
	SHL	ECX,1		; setup succeeding address
	MOV	EBP,ECX
	CMP	EBP,EDX 	; do we reach the last address?
	JNL	TST9LOP1	; go and write next base address
				; end of address line short test

	MOV	AX,1E2EH	; setup err code 2Eh for open test
	CALL	WRITERTC	; write data to RTC 17th byte of user RAM
	MOV	ESI,0		; point to physical address 0
	MOV	EAX,[ESI]	; read address 0 of SRAM
	CMP	EAX,0		; INTEL default this GTD is zero
	JNE	TST9_ERR	; something wrong, can't identfy

	MOV	AX,1E2FH	; setup error code for parrity check
	CALL	WRITERTC	; error code in RTC user RAM 17th byte
	MOV	DI,CPUST	; point to CPU status port
	MOV	AX,CS:[DI]	; read CPU status
	TEST	AX,10000000B	; keep D7, mask out other bits
	JNZ	TST9_ERR	; if not zero, we have parrity error

	MOV	ESI,40000H	; point to first address of DRAM
	MOV	EAX,0FFFFFFFFH
	MOV	[ESI],EAX
	MOV	AX,1E2EH	; setup err code 2Eh for open test
	CALL	WRITERTC	; write data to RTC 17th byte of user RAM
	MOV	ESI,0		; point to physical address 0, SRAM
	MOV	EAX,[ESI]	; read address 0 of SRAM
	CMP	EAX,0FFFFFFFFH	; INTEL default this GTD is zero
	JE	TST9_ERR	; the A18 address line open, the first
				; address of SRAM same as DRAM

	MOV	AX,1E2FH	; setup error code for parrity check
	CALL	WRITERTC	; error code in RTC user RAM 17th byte
	MOV	DI,CPUST	; point to CPU status port
	MOV	AX,CS:[DI]	; read CPU status
	TEST	AX,10000000B	; keep D7, mask out other bits
	JNZ	TST9_ERR	; if not zero, we have parrity error

	MOV	ESI,40000H	; it is the only base address
	MOV	EBP,40004H	; set succeeding address in DRAM
	MOV	ECX,00004H	; offset of succeeding address

TST9LOP6:
	MOV	AX,1E2EH	; setup err code for addr open check
	CALL	WRITERTC	; error code in RTC user RAM 17th byte
	MOV	[EBP],EBX	; write reverse data to succeeding address
	MOV	EAX,[ESI]	; read current data from current address
	CMP	EAX,0FFFFFFFFH	; verify data
	JNE	TST9_ERR	; jump if bad (no jump is fast)

	MOV	AX,1E2FH	; setup error code for parrity check
	CALL	WRITERTC	; error code in RTC user RAM 17th byte
	MOV	DI,CPUST	; point to CPU status port
	MOV	AX,CS:[DI]	; read CPU status
	TEST	AX,10000000B	; keep D7, mask out other bits
	JNZ	TST9_ERR	; if not zero, we have parrity error

	SHL	ECX,1		; get next succeeding address offset
	CMP	ECX,40000H	; do we need other algorithem?
	JNL	TST9LOP7	; ECX = 40000H now, go away
	MOV	EBP,ECX
	OR	EBP,40000H	; setup next succeeding address
	JMP	TST9LOP6	; jump if succeeding address < 8000h

TST9_ERR:
	MOV	DI,CPUFLG	; select I/O port, direct memory mapping
	MOV	CX,08H		; D3= 1, cpu led on; D3= 0, cpu led off
	MOV	CS:[DI],CX	; turn CPU LED on as an error ocurred

	MOV	DI,CPUST	; point to CPU status I/O port
	MOV	BP,CS:[DI]	; read	CPU sataus
	TEST	BP,200h 	; D9=1 loop on BIT is requested
	JZ	TST9

	JMP	$		; hang here if error

TST9LOP7:
	CMP	ECX,40000H
	JG	TST9LOP8
	SHL	ECX,1		; by pass base address
TST9LOP8:
	MOV	EBP,ECX
	SHL	EBP,1		; get next succeeding address
	CMP	EBP,EDX 	; do we reach the last address?
	JL	TST9LOP6	; go and write next succeeding address

	MOV	SI,OFFSET DRAMSIZE  ;point to string 'DRAM ???? KB'
	OR	ESI,0FFFF0000H	; convert to flat address
	MOV	ECX,ENDDRAMSIZE - DRAMSIZE
	CALL	DISPLAYMORE	;
;-------------------------------------------------------------------------+
;[Test A]  DRAM Data Fast Exchange Function TEST			  !
;									  !
;Purpose:  To test ability of all DRAM memory cells to hold a zero or a   !
;	   one and detec dynamic data/address faults.			  !
;									  !
;Description: The test is very fast because it takes advantage of various !
;	   CPU instructions. Three passes from the first to last memory   !
;	   address of DRAM are required to verify address and data faults !
;	   on both the data and parity DRAM.				  !
;									  !
; BEGIN 								  !
;     FOR DRAM first to last by 4's Do					  !
;	  write address = "FFFFFFFF"					  !
;     END (for DRAM)							  !
;									  !
;     FOR DRAM first to last by 4's Do					  !
;	  exchange "FFFFFFFF" with "00000000"				  !
;	  verify read data = "FFFFFFFF" 		   ERROR 31	  !
;	  read pattern "00000000"					  !
;	  verify read data = "00000000" 		   ERROR 31	  !
;									  !
;	  exchange "00000000" with "F4F4F4F4"				  !
;	  verify read data = "00000000" 		   ERROR 31	  !
;	  read pattern "F4F4F4F4F"					  !
;	  verify read data = "F4F4F4F4" 		   ERROR 31	  !
;     END (for DRAM)							  !
;									  !
;     Verify parity error				   ERROR 32	  !
; END									  !
;									  !
; ERRORS:								  !
; 31  Address Bus or Memory Data error					  !
;     EAX= read data	EBX= new write data	DS:SI= current or next	  !
;			EBP= last written data	       address		  !
;     17th byte of RTC user RAM = error code				  !
;									  !
; 32  Memory error, may be parity or circuit				  !
;     AX= 8, DRAM parity err						  !
;     17th byte of RTC user RAM = error code				  !
;-------------------------------------------------------------------------+

TSTA:
	CALL	LASTADDR	; upon return, EDX is last DRAM address
	MOV	EDI,40000H	; set 1st address of DRAM
	MOV	EBX,0FFFFFFFFH	; set flood data pattern
	CLD			; clear dir flag so string instr incrmnt DI
;---------------US16 can not use this instruction --------
;	REP	STOSD		; flood segment
;---------------------------------------------------------
TSTALOP1:
	MOV	[EDI],EBX	; write data to memory
	ADD	EDI,4		; point to next address
	CMP	EDI,EDX 	; max address is 83FFFFh
	JL	TSTALOP1
				; end first scan
	MOV	AX,1E31H	; setup error code 31h to RTC RAM
	CALL	WRITERTC	; point to 17th byte on user RAM
	MOV	AX,0F00H	; reset update counter in RTC RAM
	CALL	WRITERTC	; point to addr 0Fh on user RAM

	MOV	EBP,0FFFFFFFFH	; set last written pattern
	MOV	EBX,00000000H	; set revers pattern for write
	MOV	ESI,40000H	; start from 1st byte of DRAM

TSTALOP2:
	MOV	EAX,EBX 	; get revers pattern
	XCHG	[ESI],EAX	; read 'FFFFFFFF',write '00000000'
	CMP	EAX,EBP 	; verify data
	JNE	TSTAERR 	; jump if bad (no jump is fast)
;----------------------------------------------------------------------+
;	LODSD			; read '00000000' back and inc SI by 4 !
;	LOOPZ	TSTALOP4	; loop as long as CX not 0 and no error!
;	JNE	TSTAERR 	; jump if bad (no jump is fast)        !
;----------------------------------------------------------------------+
	MOV	EAX,[ESI]	; read '00000000' back

	CMP	EAX,EBX 	; verify data
	JNE	TSTAERR 	; jump if bad (no jump is fast)
	ADD	ESI,4		; point ESI to next address
	TEST	ESI,1FFFFH	; do we at 128KB boundry?
	JNZ	TSTALOP2	; no, go on r/w/r test

	MOV	AX,1E32H	; write error code 32H in RTC RAM
	CALL	CHKPARTY	; and check parity err every 256KB
	MOV	AX,1E31H	; write err code 31H as r/w/r error
	CALL	WRITERTC	; put it in RTC user RAM 17th byte
	CALL	UPDATESIZE
	CMP	ESI,EDX 	; do we reach last address?
	JL	TSTALOP2	; no,go on r/w/r test
				; end second scan
	MOV	AX,1E33H	; write err code 33H as r/w/r error
	CALL	WRITERTC	; put it in RTC user RAM 17th byte
	MOV	AX,0F00H	; reset update counter in RTC RAM
	CALL	WRITERTC	; point to addr 0Fh on user RAM

	MOV	EBP,0		; set last written pattern
	MOV	EBX,0F4F4F4F4H	; after test,all address have F4
	MOV	ESI,40000H	; start from 1st byte DRAM

TSTALOP3:
	MOV	EAX,EBX 	; get revers pattern
	XCHG	[ESI],EAX	; read '0000',write 'F4F4F4F4'
	CMP	EAX,EBP 	; verify data
	JNE	TSTAERR 	; jump if bad (no jump is fast)

;----------------------------------------------------------------------+
;	LODSD			; read 'F4F4' back and inc SI by 4     !
;	LOOPZ	TSTALOP3	; loop as long as CX not 0 and no error!
;	JNE	TSTAERR 	; jump if bad (no jump is fast)        !
;----------------------------------------------------------------------+
	MOV	EAX,[ESI]	; read 'F4F4F4F4' back
	CMP	EAX,EBX 	; verify data
	JNE	TSTAERR 	; jump if bad (no jump is fast)
	ADD	ESI,4		; point ESI to next address
	TEST	ESI,1FFFFH	; do we at 128KB boundry?
	JNZ	TSTALOP3	; no, go on r/w/r test

	MOV	AX,1E33H	; write error code 33H in RTC RAM
	CALL	CHKPARTY	; and check parity err every 256KB
	MOV	AX,1E32H	; write err code 32H as r/w/r error
	CALL	WRITERTC	; put it in RTC user RAM 17th byte

	CALL	UPDATESIZE
	CMP	ESI,EDX 	; do we reach last address?
	JL	TSTALOP3	; no,go on r/w/r test
				; end third scan

	MOV	EDI,20000H	; point to overlay SRAM address
	MOV	EBX,0F4F4F4F4H	; set flood data pattern
	CLD			; clear dir flag so string instr incrmnt DI
;---------------US16 can not use this instruction --------
;	REP	STOSD		; flood segment
;---------------------------------------------------------
TSTALOP4:
	MOV	[EDI],EBX	; write data to memory
	ADD	EDI,4		; point to next address
	CMP	EDI,40000H	; last DRAM is 3FFFFh
	JL	TSTALOP4	; cleaned overlay code area in DRAM

	MOV	DI,CPUST	; point to CPU status I/O port
	MOV	BP,CS:[DI]	; read	CPU sataus
	TEST	BP,200h 	; D9=1 loop on BIT is requested

	JNZ	start_wr	; From here to LOADER

	MOV	AX,2000h	; RTC 20h contains loopcount low byte
	CALL	READRTC
	MOV	BL,AL		; save loopcount low byte
	MOV	AX,2100h	; RTC 21h contains loopcount hi byte
	CALL	READRTC
	MOV	BH,AL		; save loopcount hi byte

	INC	BX		; increase loop count
	MOV	AH,20h		; RTC RAM 20h contains loopcount low byte
	MOV	AL,BL		; update loopcount
	CALL	WRITERTC
	MOV	AH,21h		; RTC RAM 21h contains loopcount hi byte
	MOV	AL,BH		; update loopcount
	CALL	WRITERTC

	MOV	SI,OFFSET LOOPCONT
	OR	ESI,0FFFF0000H	; ESI point to string first address
	MOV	ECX,ENDLOOPCONT - LOOPCONT	;count bytes
	CALL	DISPLAYMORE	; position cursor to least significant digit
	MOV	AX,BX		; put loopcount to AX for display
	CALL	UPDATENUM	; display loopcount at terminal srceen

	MOV	ECX,0FFFFFFFh	; time delay in order user
	LOOP	$		; can see the last DRAM size on screen
	MOV	SI,OFFSET DRAMCLEN
	OR	ESI,0FFFF0000H	; ESI point to string first address
	MOV	ECX,ENDDRAMCLEN - DRAMCLEN	;count bytes
	CALL	DISPLAYMORE	; clean msg DRAM ???? KB on screen
	JMP	TST6		; let's go back in loop

TSTAERR:
	MOV	EDI,CPUFLG	; select I/O port, direct memory mapping
	MOV	CX,08H		; D3= 1, cpu led on; D3= 0, cpu led off
	MOV	CS:[DI],CX	; turn CPU LED on

	MOV	DI,CPUST	; point to CPU status I/O port
	MOV	BP,CS:[DI]	; read	CPU sataus
	TEST	BP,200h 	; D9=1 loop on BIT is requested
	JZ	TSTA

	JMP	$		; hang here if error

;********* all these subroutins are vailid only in Protect Mode *******
;-------->>>>>------------------+
LASTADDR	PROC	NEAR	;
;-------------------------------+---------------------------------------+
;on entry: none 							!
;on exit : EDX is the byte number of DRAM, or the last DRAM address + 1 !
;regs destroied: none							!
;-------------------------------+---------------------------------------+

	PUSH	EBP
	MOV	EBP,RTCLOCK	; point to RTC address select port
	MOV	EDX,0Eh 	; point to 1st byte on user RAM
	MOV	[EBP],DL	;
	MOV	EBP,RTCLOC2	; point to RTC data r/w port
	MOV	DL,[EBP]	; read DRAM size from RTC RAM
	SHL	EDX,20		; convert size to flat address
	ADD	EDX,40000H	; plus 1st byte offset
	POP	EBP
	RET
;-------------------------------+
LASTADDR	ENDP		;
;--------<<<<<------------------+
;-------->>>>>------------------+
WRITERTC	PROC	NEAR	;
;-------------------------------+---------------------------------------+
;on entry: AH= RTC user RAM address (0E-FF),				!
;	   AL= data to be written to RTC user RAM			!
;on exit : none 							!
;regs destroied: none							!
;-------------------------------+---------------------------------------+
	PUSH	EBP		;
	MOV	EBP,RTCLOCK	; point to RTC address select port
	MOV	[EBP],AH	; AH is the user ram address
	MOV	EBP,RTCLOC2	; point to RTC data r/w port
	MOV	[EBP],AL	; write data code to RTC RAM
	POP	EBP		;
	RET			;
;-------------------------------+
WRITERTC	ENDP		;
;--------<<<<<------------------+---------------------------------------+
;The RTC user RAM address starts from 0Eh to FFh, total 50 bytes.	*
;Diagnostic reserved following address: 				*
;	[0Eh]= the DRAM memory size in unit of MB,bootstrap will read it*
;	[0Fh]= external counter for update DRAM size shown on screen	*
;	[1Eh]= error code store here during protect mode		*
;	[1Fh]= MNI flag; 0= no NMI, 1= NMI occured during diagnostics	*
;	[20h]= loop count low byte					*
;	[21h]= loop count hi byte					*
;-------->>>>>------------------+---------------------------------------+
READRTC 	PROC	NEAR	;
;-------------------------------+---------------------------------------+
;on entry: AH is the address of RTC user RAM,				!
;on exit : AL is the data read from RTC user RAM at address 'AH'	!
;regs destroied: none							!
;-------------------------------+---------------------------------------+
	PUSH	EBP		;
	MOV	EBP,RTCLOCK	; point to RTC address select port
	MOV	[EBP],AH	; AH is the user ram address
	MOV	EBP,RTCLOC2	; point to RTC data r/w port
	MOV	AL,[EBP]	; read data code from RTC RAM
	POP	EBP		;
	RET			;
;-------------------------------+
READRTC 	ENDP		;
;--------<<<<<------------------+
;-------->>>>>------------------+
DISPLAYMORE	PROC	NEAR	;
;-------------------------------+---------------------------------------+
;on entry: ESI is point to the first byte of string			;
;	   ECX is the number of bytes in the string			;
;on exit : none 							;
;reg destroyed: none							;
;-----------------------------------------------------------------------+
	PUSH	EAX
DISPLAYMLOP:
	MOV	AL,[ESI]	;
	CALL	DISPLAYONE
	INC	ESI
	LOOP	DISPLAYMLOP
	POP	EAX
	RET
;-------------------------------+
DISPLAYMORE	ENDP		;
;--------<<<<<------------------+
;-------->>>>>------------------+
DISPLAYONE	PROC	NEAR	;
;-------------------------------+---------------------------------------+
;on entry: AL is the ASCII code to be displayed 			;
;on exit : none 							;
;reg destroyed: none							;
;-----------------------------------------------------------------------+
	CALL	CHK5USPULS	; delay 5useconds before next character
	CALL	CHKIORDY	; be sure CRT is ready for next send
	PUSH	EDI
	MOV	EDI,_OBS	; point to output data bus strobe
	MOV	[EDI],AL	; send out ascii
	POP	EDI
	RET
;-------------------------------+
DISPLAYONE	ENDP		;
;--------<<<<<------------------+
;-------->>>>>------------------+
ENABLECRT	PROC	NEAR	;
;-------------------------------+---------------------------------------+
;on entry: AL is the device address to be enabled, CRT=05, PRT= 	;
;on exit : none
;reg destroyed: none							;
;-----------------------------------------------------------------------+
	PUSH	DI
	MOV	DI,_ABS 	;point to I/O address bus strobe
	MOV	AL,5		;device CRT's address is 05H
	MOV	CS:[DI],AL
	CALL	CHK5USPULS	;wait for two 5usec pulses
	CALL	CHKIORDY	;make sure I/O bus is ready
	POP	DI
	RET
;-------------------------------+
ENABLECRT	ENDP		;
;--------<<<<<------------------+
;-------->>>>>------------------+
CHK5USPULS	PROC	NEAR	;
;-------------------------------+---------------------------------------+
;purpose: check CPUST D0= 1 twice in order to get min dealy 5usec	;
;on entry: none 							;
;on exit : none 							;
;regs destroy: none							;
;-----------------------------------------------------------------------+
	PUSH	DI
	PUSH	AX
	MOV	DI,8002H	;point tp CPU I/O status port
CHK5USPULSA:
	MOV	AL,BYTE PTR CS:[DI]
	TEST	AL,01H		;D0=1, 5usec pulse for ABS/, OBS/, CBS/
	JZ	CHK5USPULSA	;try again, be sure got first pulse
CHK5USPULSB:
	MOV	AL,BYTE PTR CS:[DI]
	TEST	AL,01H		;D0=1, 5usec pulse for ABS/, OBS/, CBS/
	JZ	CHK5USPULSB	;try again in order to get second pulse
	POP	AX
	POP	DI
	RET
;-------------------------------+
CHK5USPULS	ENDP		;
;--------<<<<<------------------+
;-------->>>>>------------------+
CHKIORDY	PROC	NEAR	;
;-------------------------------+---------------------------------------+
;purpose: check CPUST D3=1 as I/O ready 				;
;on entry: none 							;
;on exit : none 							;
;regs destroy: none							;
;-----------------------------------------------------------------------+
	PUSH	DI
	MOV	DI,CPUST
	TEST	BYTE PTR CS:[DI],08H ;D3=0,I/O not ready;D3=1,I/O raedy
	POP	DI
	JZ	CHKIORDY	; if not ready, let's wait
	RET
;-------------------------------+
CHKIORDY	ENDP		;
;--------<<<<<------------------+
;-------->>>>>------------------+
CHKPARTY	PROC	NEAR	;
;-------------------------------+---------------------------------------+
;purpose: verify whether parity err occurred or not. if occurred, loop. !
;on entry: AH is the RTC RAM address, AL is error code			!
;on exit : none, if no parity err. otherwise loop within this subroutine!
;regs destroy: none							!
;-----------------------------------------------------------------------+
	CALL	WRITERTC	; write err code first
	PUSH	EAX
	PUSH	EBX
	MOV	BX,CPUST	; point to CPU status port
	MOV	AX,CS:[BX]	; read CPU status
	AND	AX,11000000B	; keep D6 & D7, mask out other bits
	CMP	AX,0		; D6 & D7 should be all zero
	JE	CHKPRTYA	; if no parity err, go exit
	JMP	$		; if parity err, stop here
CHKPRTYA:
	POP	EBX
	POP	EAX
	RET
;-------------------------------+
CHKPARTY	ENDP		;
;--------<<<<<------------------+
;-------->>>>>------------------+
UPDATENUM	PROC	NEAR	;
;-------------------------------+---------------------------------------+
;purpose: convert word hex number to decimal and display it.		!
;on entry: AX contains hex number, cursor points to least signif digit. !
;on exit : none 							!
;regs destroy: none							!
;-----------------------------------------------------------------------+
	PUSH	EAX
	PUSH	EBX
	PUSH	ECX
	PUSH	EDX

	MOV	EBX,10		; set divisor
	MOV	ECX,4		; maximun 5 digits, there are 4 times DIV
UPDATENUM1:
	MOV	EDX,0		; clear remainder
	DIV	BX		; (DX+AX)/BX=AX+DX
	ADD	DL,30h		; convert remainder to ascii
	XCHG	AX,DX		; put ascii to AX for display
	CALL	DISPLAYONE	; display this ascii to screen

	MOV	AL,08h		; ascii 08h is back space
	CALL	DISPLAYONE	; mov cursor back
	MOV	AL,08h		; ascii 08h is back space
	CALL	DISPLAYONE	; set cursor to next digit position

	CMP	EDX,0		; is quotient zero?
	JE	UPDATENUM2	; it is zero, no more display

	MOV	AX,DX		; restore quotien for next div
	LOOP	UPDATENUM1	; continue division

	ADD	AL,30h		; convert last quotient to ascii
	CALL	DISPLAYONE	; display this ascii to screen

UPDATENUM2:

	POP	EDX
	POP	ECX
	POP	EBX
	POP	EAX
	RET
;-------------------------------+
UPDATENUM	ENDP		;
;--------<<<<<------------------+
;-------->>>>>------------------+
UPDATESIZE	PROC	NEAR	;
;-------------------------------+---------------------------------------+
;purpose: show dram memory size on screen in 128KB increasment		;
;on entry:none on register, but cursor point to least significant digit ;
;on exit :none, but cursor point to least significsnt digit		;
;regs destroy: none							;
;-----------------------------------------------------------------------+
	PUSH	ESI
	PUSH	EDX
	PUSH	ECX
	PUSH	EAX

	MOV	EAX,ESI 	; save surrent address
	MOV	SI,OFFSET DRAMLOOP
	OR	ESI,0FFFF0000H	; ESI point to string first address
	MOV	ECX,ENDDRAMLOOP - DRAMLOOP	;count bytes
	CALL	DISPLAYMORE	; reset cursor at least significant digit

	MOV	ECX,1024	; divided by 1024
	SUB	EAX,40000H	; deduct sram size
	MOV	EDX,0		; clear remainder
	DIV	ECX		; (EDX+EAX)/ECX= EAX + EDX
	CALL	UPDATENUM	; convert size to decimal and display

	POP	EAX
	POP	ECX
	POP	EDX
	POP	ESI
	RET
;-------------------------------+
UPDATESIZE	ENDP		;
;--------<<<<<------------------+


START_WR:
	mov	bx,flat_des -temp_gdt
	mov	ds,bx			    ;	ds,es,ss all point to
	mov	es,bx			    ;	temp. gdt [1]
	mov	ss,bx
	mov	esp,offset end_temp_gdt
;
	mov	edi,bootid
	mov	bl,byte ptr [edi]	 ;   bx = boot id
	mov	bh,byte ptr [edi+2]

	mov	edi,rtclock	; point to RTC address select port
	mov	ah,0fh		; point to 2th byte on user RAM
	mov	[edi],ah      ;
	mov	edi,rtcloc2	; point to RTC data r/w port
	mov	[edi],bh      ; write high byte to RTC RAM

	mov	edi,rtclock	; point to RTC address select port
	mov	ah,10h		; point to 3th byte on user RAM
	mov	[edi],ah      ;
	mov	edi,rtcloc2	; point to RTC data r/w port
	mov	[edi],bl      ; write low byte	to RTC RAM
;
; copy eprom gdt to ram
;
copy2:
	mov	esi,dword ptr gdt_eprom +2
	mov	edi,gdtbase
	mov	cx,word ptr gdt_eprom +0
	inc	cx
	shr	cx,1
	cld
	rep	movs word ptr es:[edi],word ptr ds:[esi]
;
; switch to ram gdt
;
	lgdt	gdt_pword
;
	mov	bx,gdt_alias
	mov	ds,bx
;
; copy eprom tss to ram
;
	mov	bx,init_tss_a
	mov	es,bx
	mov	bx,init_tss
	lar	dx,bx
	mov	[bx].access,ds_access
	mov	fs,bx
	xor	si,si
	xor	di,di
	mov	cx,[bx].lim_0_15
	inc	cx
	rep	movs byte ptr es:[di],byte ptr fs:[si]
	mov	[bx].access,dh
; alter the base address of the initial task's tss
	mov	ax,init_tss_a_offset.bas_0_15
	mov	init_tss_offset.bas_0_15,ax
	mov	al,init_tss_a_offset.bas_16_23
	mov	init_tss_offset.bas_16_23,al
	mov	al,init_tss_a_offset.bas_24_31
	mov	init_tss_offset.bas_24_31,al
;
; change init_tss_a to form a save area for tss on the first task switch
;
	mov	bx,init_tss_a
	mov	word ptr [bx].bas_0_15,1A80h
	mov	[bx].bas_16_23,05h
	mov	[bx].bas_24_31,0
	mov	[bx].access,tss_access
	mov	[bx].gran,0
	ltr	bx
;
; copy eprom ldt to ram
;
	mov	bx,init_ldt_a
	mov	es,bx
	mov	ah,[bx].bas_24_31
	mov	al,[bx].bas_16_23
	shl	eax,16
	mov	ax,[bx].bas_0_15
;
	mov	bx,init_ldt
	lar	dx,bx
	mov	[bx].access,ds_access
	mov	fs,bx
	xor	si,si
	xor	di,di
	mov	cx,[bx].lim_0_15
	inc	cx
	rep	movs byte ptr es:[di],byte ptr fs:[si]
	mov	[bx].access,dh
;
; alter the base address of ldt's alias
	mov	es:[init_ldt_alias].bas_0_15,ax
	shr	eax,16
	mov	es:[init_ldt_alias].bas_16_23,al
	mov	es:[init_ldt_alias].bas_24_31,ah
;
; alter the base address of initial task's ldt
	mov	ax,init_ldt_a_offset.bas_0_15
	mov	init_ldt_offset.bas_0_15,ax
	mov	al,init_ldt_a_offset.bas_16_23
	mov	init_ldt_offset.bas_16_23,al
	mov	al,init_ldt_a_offset.bas_24_31
	mov	init_ldt_offset.bas_24_31,al
;
; now gdt, idt, initial tss and initial ldt are all set up
; start the first task
	jmp	entry_pointer
;
	org	adjust3
	jmp	start_wr
;
	org	adjust2
	jmp	start

reset_code	ends
	end
